/****************************************************************************
** Meta object code from reading C++ file 'server.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/server.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'server.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7CServerE_t {};
} // unnamed namespace

template <> constexpr inline auto CServer::qt_create_metaobjectdata<qt_meta_tag_ZN7CServerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CServer",
        "Started",
        "",
        "Stopped",
        "ClientDisconnected",
        "iChID",
        "SvrRegStatusChanged",
        "AudioFrame",
        "stChName",
        "CHostAddress",
        "RecHostAddr",
        "iNumAudChan",
        "CVector<int16_t>",
        "vecsData",
        "CLVersionAndOSReceived",
        "InetAddr",
        "COSUtil::EOpSystemType",
        "eOSType",
        "strVersion",
        "RestartRecorder",
        "StopRecorder",
        "RecordingSessionStarted",
        "sessionDir",
        "EndRecorderThread",
        "OnTimer",
        "OnNewConnection",
        "iTotChans",
        "OnServerFull",
        "OnSendCLProtMessage",
        "CVector<uint8_t>",
        "vecMessage",
        "OnProtocolCLMessageReceived",
        "iRecID",
        "vecbyMesBodyData",
        "OnProtocolMessageReceived",
        "iRecCounter",
        "OnCLPingReceived",
        "iMs",
        "OnCLPingWithNumClientsReceived",
        "OnCLSendEmptyMes",
        "TargetInetAddr",
        "OnCLReqServerList",
        "OnCLReqVersionAndOS",
        "OnCLReqConnClientsList",
        "OnCLRegisterServerReceived",
        "LInetAddr",
        "CServerCoreInfo",
        "ServerInfo",
        "OnCLRegisterServerExReceived",
        "OnCLRegisterServerResp",
        "ESvrRegResult",
        "eResult",
        "OnCLUnregisterServerReceived",
        "OnCLDisconnection",
        "OnAboutToQuit",
        "OnHandledSignal",
        "sigNum"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'Started'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'Stopped'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ClientDisconnected'
        QtMocHelpers::SignalData<void(const int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Signal 'SvrRegStatusChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'AudioFrame'
        QtMocHelpers::SignalData<void(const int, const QString, const CHostAddress, const int, const CVector<int16_t>)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { QMetaType::QString, 8 }, { 0x80000000 | 9, 10 }, { QMetaType::Int, 11 },
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'CLVersionAndOSReceived'
        QtMocHelpers::SignalData<void(CHostAddress, COSUtil::EOpSystemType, QString)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 15 }, { 0x80000000 | 16, 17 }, { QMetaType::QString, 18 },
        }}),
        // Signal 'RestartRecorder'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'StopRecorder'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'RecordingSessionStarted'
        QtMocHelpers::SignalData<void(QString)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 22 },
        }}),
        // Signal 'EndRecorderThread'
        QtMocHelpers::SignalData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnTimer'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnNewConnection'
        QtMocHelpers::SlotData<void(int, int, CHostAddress)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { QMetaType::Int, 26 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'OnServerFull'
        QtMocHelpers::SlotData<void(CHostAddress)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'OnSendCLProtMessage'
        QtMocHelpers::SlotData<void(CHostAddress, CVector<uint8_t>)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 15 }, { 0x80000000 | 29, 30 },
        }}),
        // Slot 'OnProtocolCLMessageReceived'
        QtMocHelpers::SlotData<void(int, CVector<uint8_t>, CHostAddress)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 32 }, { 0x80000000 | 29, 33 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'OnProtocolMessageReceived'
        QtMocHelpers::SlotData<void(int, int, CVector<uint8_t>, CHostAddress)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 35 }, { QMetaType::Int, 32 }, { 0x80000000 | 29, 33 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'OnCLPingReceived'
        QtMocHelpers::SlotData<void(CHostAddress, int)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 15 }, { QMetaType::Int, 37 },
        }}),
        // Slot 'OnCLPingWithNumClientsReceived'
        QtMocHelpers::SlotData<void(CHostAddress, int, int)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 15 }, { QMetaType::Int, 37 }, { QMetaType::Int, 2 },
        }}),
        // Slot 'OnCLSendEmptyMes'
        QtMocHelpers::SlotData<void(CHostAddress)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 40 },
        }}),
        // Slot 'OnCLReqServerList'
        QtMocHelpers::SlotData<void(CHostAddress)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 15 },
        }}),
        // Slot 'OnCLReqVersionAndOS'
        QtMocHelpers::SlotData<void(CHostAddress)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 15 },
        }}),
        // Slot 'OnCLReqConnClientsList'
        QtMocHelpers::SlotData<void(CHostAddress)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 15 },
        }}),
        // Slot 'OnCLRegisterServerReceived'
        QtMocHelpers::SlotData<void(CHostAddress, CHostAddress, CServerCoreInfo)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 15 }, { 0x80000000 | 9, 45 }, { 0x80000000 | 46, 47 },
        }}),
        // Slot 'OnCLRegisterServerExReceived'
        QtMocHelpers::SlotData<void(CHostAddress, CHostAddress, CServerCoreInfo, COSUtil::EOpSystemType, QString)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 15 }, { 0x80000000 | 9, 45 }, { 0x80000000 | 46, 47 }, { 0x80000000 | 16, 2 },
            { QMetaType::QString, 18 },
        }}),
        // Slot 'OnCLRegisterServerResp'
        QtMocHelpers::SlotData<void(CHostAddress, ESvrRegResult)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 2 }, { 0x80000000 | 50, 51 },
        }}),
        // Slot 'OnCLUnregisterServerReceived'
        QtMocHelpers::SlotData<void(CHostAddress)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 15 },
        }}),
        // Slot 'OnCLDisconnection'
        QtMocHelpers::SlotData<void(CHostAddress)>(53, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 15 },
        }}),
        // Slot 'OnAboutToQuit'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnHandledSignal'
        QtMocHelpers::SlotData<void(int)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 56 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CServer, qt_meta_tag_ZN7CServerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CServer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7CServerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7CServerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7CServerE_t>.metaTypes,
    nullptr
} };

void CServer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CServer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->Started(); break;
        case 1: _t->Stopped(); break;
        case 2: _t->ClientDisconnected((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->SvrRegStatusChanged(); break;
        case 4: _t->AudioFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<CVector<int16_t>>>(_a[5]))); break;
        case 5: _t->CLVersionAndOSReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<COSUtil::EOpSystemType>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 6: _t->RestartRecorder(); break;
        case 7: _t->StopRecorder(); break;
        case 8: _t->RecordingSessionStarted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->EndRecorderThread(); break;
        case 10: _t->OnTimer(); break;
        case 11: _t->OnNewConnection((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[3]))); break;
        case 12: _t->OnServerFull((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 13: _t->OnSendCLProtMessage((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CVector<uint8_t>>>(_a[2]))); break;
        case 14: _t->OnProtocolCLMessageReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CVector<uint8_t>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[3]))); break;
        case 15: _t->OnProtocolMessageReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<CVector<uint8_t>>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[4]))); break;
        case 16: _t->OnCLPingReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 17: _t->OnCLPingWithNumClientsReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 18: _t->OnCLSendEmptyMes((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 19: _t->OnCLReqServerList((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 20: _t->OnCLReqVersionAndOS((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 21: _t->OnCLReqConnClientsList((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 22: _t->OnCLRegisterServerReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<CServerCoreInfo>>(_a[3]))); break;
        case 23: _t->OnCLRegisterServerExReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<CServerCoreInfo>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<COSUtil::EOpSystemType>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5]))); break;
        case 24: _t->OnCLRegisterServerResp((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ESvrRegResult>>(_a[2]))); break;
        case 25: _t->OnCLUnregisterServerReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 26: _t->OnCLDisconnection((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 27: _t->OnAboutToQuit(); break;
        case 28: _t->OnHandledSignal((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CVector<int16_t> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CServer::*)()>(_a, &CServer::Started, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CServer::*)()>(_a, &CServer::Stopped, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CServer::*)(const int )>(_a, &CServer::ClientDisconnected, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CServer::*)()>(_a, &CServer::SvrRegStatusChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (CServer::*)(const int , const QString , const CHostAddress , const int , const CVector<int16_t> )>(_a, &CServer::AudioFrame, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (CServer::*)(CHostAddress , COSUtil::EOpSystemType , QString )>(_a, &CServer::CLVersionAndOSReceived, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (CServer::*)()>(_a, &CServer::RestartRecorder, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (CServer::*)()>(_a, &CServer::StopRecorder, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (CServer::*)(QString )>(_a, &CServer::RecordingSessionStarted, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (CServer::*)()>(_a, &CServer::EndRecorderThread, 9))
            return;
    }
}

const QMetaObject *CServer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CServer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7CServerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "CServerSlots<150>"))
        return static_cast< CServerSlots<150>*>(this);
    return QObject::qt_metacast(_clname);
}

int CServer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    return _id;
}

// SIGNAL 0
void CServer::Started()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void CServer::Stopped()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void CServer::ClientDisconnected(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void CServer::SvrRegStatusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void CServer::AudioFrame(const int _t1, const QString _t2, const CHostAddress _t3, const int _t4, const CVector<int16_t> _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 5
void CServer::CLVersionAndOSReceived(CHostAddress _t1, COSUtil::EOpSystemType _t2, QString _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3);
}

// SIGNAL 6
void CServer::RestartRecorder()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void CServer::StopRecorder()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void CServer::RecordingSessionStarted(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void CServer::EndRecorderThread()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
