/********************************************************************************
** Form generated from reading UI file 'clientsettingsdlgbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CLIENTSETTINGSDLGBASE_H
#define UI_CLIENTSETTINGSDLGBASE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "util.h"

QT_BEGIN_NAMESPACE

class Ui_CClientSettingsDlgBase
{
public:
    QHBoxLayout *horizontalLayout_7;
    QTabWidget *tabSettings;
    QWidget *tab_0_User;
    QVBoxLayout *verticalLayout_16;
    QHBoxLayout *horizontalLayout_10;
    QSpacerItem *horizontalSpacer;
    QVBoxLayout *verticalLayout_12;
    QSpacerItem *verticalSpacer_8;
    QGroupBox *grbMusicien;
    QVBoxLayout *verticalLayout_10;
    QHBoxLayout *horizontalLayout_9;
    QVBoxLayout *verticalLayout_8;
    QLabel *plblAlias;
    QLabel *plblInstrument;
    QLabel *plblCountry;
    QLabel *plblCity;
    QLabel *plblSkill;
    QVBoxLayout *verticalLayout_9;
    QLineEdit *pedtAlias;
    QComboBox *pcbxInstrument;
    QComboBox *pcbxCountry;
    QLineEdit *pedtCity;
    QComboBox *pcbxSkill;
    QSpacerItem *verticalSpacer_7;
    QGroupBox *gbxUserInterface;
    QVBoxLayout *verticalLayout_11;
    QHBoxLayout *horizontalLayout_3;
    QVBoxLayout *verticalLayout_5;
    QLabel *lblSkin;
    QLabel *lblMeterStyle;
    QLabel *lblLanguage;
    QLabel *lblMixerRows;
    QLabel *lblAudioAlerts;
    QVBoxLayout *verticalLayout_7;
    QComboBox *cbxSkin;
    QComboBox *cbxMeterStyle;
    CLanguageComboBox *cbxLanguage;
    QSpinBox *spnMixerRows;
    QCheckBox *chbAudioAlerts;
    QSpacerItem *verticalSpacer_9;
    QSpacerItem *horizontalSpacer_2;
    QWidget *tab_1_AudioNet;
    QHBoxLayout *horizontalLayout_6;
    QHBoxLayout *horizontalLayout_5;
    QVBoxLayout *verticalLayout_3;
    QLabel *lblSoundcardDevice;
    QComboBox *cbxSoundcard;
    QSpacerItem *verticalSpacer_3;
    QPushButton *butDriverSetup;
    QSpacerItem *verticalSpacer_5;
    QFrame *FrameSoundcardChannelSelection;
    QVBoxLayout *_2;
    QLabel *lblInChannelMapping;
    QHBoxLayout *_3;
    QVBoxLayout *_4;
    QLabel *lblLInChan;
    QLabel *lblRInChan;
    QVBoxLayout *_5;
    QComboBox *cbxLInChan;
    QComboBox *cbxRInChan;
    QLabel *lblOutChannelMapping;
    QHBoxLayout *_6;
    QVBoxLayout *_7;
    QLabel *lblLOutChan;
    QLabel *lblROutChan;
    QVBoxLayout *_8;
    QComboBox *cbxLOutChan;
    QComboBox *cbxROutChan;
    QSpacerItem *verticalSpacer_6;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout_2;
    QLabel *lblAudioChannels;
    QLabel *lblAudioQuality;
    QVBoxLayout *verticalLayout;
    QComboBox *cbxAudioChannels;
    QComboBox *cbxAudioQuality;
    QSpacerItem *verticalSpacer_10;
    QGroupBox *grbSoundCrdBufDelay;
    QVBoxLayout *_9;
    QRadioButton *rbtBufferDelayPreferred;
    QRadioButton *rbtBufferDelayDefault;
    QRadioButton *rbtBufferDelaySafe;
    QSpacerItem *horizontalSpacer_3;
    QVBoxLayout *verticalLayout_13;
    QGroupBox *grbJitterBuffer;
    QVBoxLayout *_15;
    QCheckBox *chbAutoJitBuf;
    QHBoxLayout *_16;
    QLabel *lblNetBufLabel;
    QLabel *lblNetBufServerLabel;
    QHBoxLayout *_17;
    QLabel *lblNetBuf;
    QLabel *lblNetBufServer;
    QHBoxLayout *_18;
    QSpacerItem *spacerItem;
    QSlider *sldNetBuf;
    QSpacerItem *spacerItem1;
    QSlider *sldNetBufServer;
    QCheckBox *chbSmallNetworkBuffers;
    QGroupBox *grbUpstreamValue;
    QVBoxLayout *verticalLayout_14;
    QGridLayout *_10;
    QSpacerItem *horizontalSpacer_9;
    QLabel *lblUpstreamUnit;
    QLabel *lblUpstreamValue;
    QSpacerItem *horizontalSpacer_10;
    QSpacerItem *horizontalSpacer_5;
    QWidget *tab_2_advanced;
    QVBoxLayout *verticalLayout_15;
    QVBoxLayout *verticalLayout_6;
    QHBoxLayout *horizontalLayout_8;
    QSpacerItem *horizontalSpacer_6;
    QVBoxLayout *verticalLayout_4;
    QSpacerItem *verticalSpacer_2;
    QLabel *lblCustomDirectories;
    QHBoxLayout *hboxLayout;
    QComboBox *cbxCustomDirectories;
    QToolButton *tbtDeleteCustomDirectory;
    QSpacerItem *verticalSpacer_4;
    QLabel *lblNewClientLevel;
    QHBoxLayout *horizontalLayout_4;
    QLineEdit *edtNewClientLevel;
    QLabel *lblPercentUnit;
    QSpacerItem *verticalSpacer_11;
    QLabel *lblInputBoost;
    QComboBox *cbxInputBoost;
    QSpacerItem *verticalSpacer_13;
    QCheckBox *chbDetectFeedback;
    QSpacerItem *verticalSpacer_15;
    QSpacerItem *horizontalSpacer_4;
    QHBoxLayout *horizontalLayout_12;
    QSpacerItem *horizontalSpacer_7;
    QGroupBox *groupBox;
    QHBoxLayout *horizontalLayout_2;
    QVBoxLayout *_19;
    QLabel *lblAudioPan;
    QHBoxLayout *_20;
    QSpacerItem *spacerItem2;
    QSlider *sldAudioPan;
    QSpacerItem *spacerItem3;
    QLabel *lblAudioPanValue;
    QSpacerItem *horizontalSpacer_8;
    QSpacerItem *verticalSpacer;
    QWidget *tab_3_midi;
    QVBoxLayout *verticalLayout_18;
    QWidget *wdgMidiDeviceContainer;
    QHBoxLayout *horizontalLayout_midiDevice;
    QGroupBox *grbMidiControls;
    QGridLayout *gridLayout;
    QHBoxLayout *horizontalLayout_17;
    QLabel *lblChannel;
    QComboBox *cbxChannel;
    QSpacerItem *horizontalSpacer_15;
    QCheckBox *chbMIDIPickupMode;
    QSpacerItem *horizontalSpacer_18;
    QGroupBox *grbMidiMuteMyself;
    QHBoxLayout *horizontalLayout_20;
    QHBoxLayout *horizontalLayout_16;
    QLabel *lblMuteMyselfCC;
    QSpinBox *spnMuteMyself;
    QPushButton *butLearnMuteMyself;
    QSpacerItem *horizontalSpacer_17;
    QSpacerItem *verticalSpacer_18;
    QSpacerItem *verticalSpacer_17;
    QGridLayout *gridLayoutMain;
    QSpacerItem *verticalSpacer_14;
    QGroupBox *grbMidiFader;
    QHBoxLayout *horizontalLayout_19;
    QHBoxLayout *horizontalLayout_11;
    QLabel *lblFaderOffset;
    QSpinBox *spnFaderOffset;
    QPushButton *butLearnFaderOffset;
    QSpacerItem *horizontalSpacer_11;
    QLabel *lblFaderCount;
    QSpinBox *spnFaderCount;
    QGroupBox *grbMidiMute;
    QHBoxLayout *horizontalLayout_18;
    QHBoxLayout *horizontalLayout_15;
    QLabel *lblMuteOffset;
    QSpinBox *spnMuteOffset;
    QPushButton *butLearnMuteOffset;
    QSpacerItem *horizontalSpacer_14;
    QLabel *lblMuteCount;
    QSpinBox *spnMuteCount;
    QSpacerItem *verticalSpacer_12;
    QGroupBox *grbMidiSolo;
    QHBoxLayout *horizontalLayout_21;
    QHBoxLayout *horizontalLayout_14;
    QLabel *lblSoloOffset;
    QSpinBox *spnSoloOffset;
    QPushButton *butLearnSoloOffset;
    QSpacerItem *horizontalSpacer_13;
    QLabel *lblSoloCount;
    QSpinBox *spnSoloCount;
    QGroupBox *grbMidiPan;
    QHBoxLayout *horizontalLayout_22;
    QHBoxLayout *horizontalLayout_13;
    QLabel *lblPanOffset;
    QSpinBox *spnPanOffset;
    QPushButton *butLearnPanOffset;
    QSpacerItem *horizontalSpacer_12;
    QLabel *lblPanCount;
    QSpinBox *spnPanCount;
    QSpacerItem *verticalSpacer_16;
    QHBoxLayout *horizontalLayout_23;
    QLabel *lblMidiDevice;
    QComboBox *cbxMidiDevice;
    QSpacerItem *horizontalSpacer_16;
    QSpacerItem *verticalSpacer_19;

    void setupUi(QDialog *CClientSettingsDlgBase)
    {
        if (CClientSettingsDlgBase->objectName().isEmpty())
            CClientSettingsDlgBase->setObjectName("CClientSettingsDlgBase");
        CClientSettingsDlgBase->resize(534, 591);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/png/main/res/fronticon.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        CClientSettingsDlgBase->setWindowIcon(icon);
        CClientSettingsDlgBase->setSizeGripEnabled(true);
        horizontalLayout_7 = new QHBoxLayout(CClientSettingsDlgBase);
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        tabSettings = new QTabWidget(CClientSettingsDlgBase);
        tabSettings->setObjectName("tabSettings");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(tabSettings->sizePolicy().hasHeightForWidth());
        tabSettings->setSizePolicy(sizePolicy);
        tabSettings->setUsesScrollButtons(true);
        tab_0_User = new QWidget();
        tab_0_User->setObjectName("tab_0_User");
        verticalLayout_16 = new QVBoxLayout(tab_0_User);
        verticalLayout_16->setObjectName("verticalLayout_16");
        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer);

        verticalLayout_12 = new QVBoxLayout();
        verticalLayout_12->setObjectName("verticalLayout_12");
        verticalSpacer_8 = new QSpacerItem(20, 13, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);

        verticalLayout_12->addItem(verticalSpacer_8);

        grbMusicien = new QGroupBox(tab_0_User);
        grbMusicien->setObjectName("grbMusicien");
        verticalLayout_10 = new QVBoxLayout(grbMusicien);
        verticalLayout_10->setObjectName("verticalLayout_10");
        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        verticalLayout_8 = new QVBoxLayout();
        verticalLayout_8->setObjectName("verticalLayout_8");
        plblAlias = new QLabel(grbMusicien);
        plblAlias->setObjectName("plblAlias");

        verticalLayout_8->addWidget(plblAlias);

        plblInstrument = new QLabel(grbMusicien);
        plblInstrument->setObjectName("plblInstrument");

        verticalLayout_8->addWidget(plblInstrument);

        plblCountry = new QLabel(grbMusicien);
        plblCountry->setObjectName("plblCountry");

        verticalLayout_8->addWidget(plblCountry);

        plblCity = new QLabel(grbMusicien);
        plblCity->setObjectName("plblCity");

        verticalLayout_8->addWidget(plblCity);

        plblSkill = new QLabel(grbMusicien);
        plblSkill->setObjectName("plblSkill");

        verticalLayout_8->addWidget(plblSkill);


        horizontalLayout_9->addLayout(verticalLayout_8);

        verticalLayout_9 = new QVBoxLayout();
        verticalLayout_9->setObjectName("verticalLayout_9");
        pedtAlias = new QLineEdit(grbMusicien);
        pedtAlias->setObjectName("pedtAlias");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(pedtAlias->sizePolicy().hasHeightForWidth());
        pedtAlias->setSizePolicy(sizePolicy1);
        pedtAlias->setMinimumSize(QSize(150, 0));

        verticalLayout_9->addWidget(pedtAlias);

        pcbxInstrument = new QComboBox(grbMusicien);
        pcbxInstrument->setObjectName("pcbxInstrument");
        pcbxInstrument->setMinimumSize(QSize(150, 0));

        verticalLayout_9->addWidget(pcbxInstrument);

        pcbxCountry = new QComboBox(grbMusicien);
        pcbxCountry->setObjectName("pcbxCountry");
        pcbxCountry->setMinimumSize(QSize(150, 0));

        verticalLayout_9->addWidget(pcbxCountry);

        pedtCity = new QLineEdit(grbMusicien);
        pedtCity->setObjectName("pedtCity");
        sizePolicy1.setHeightForWidth(pedtCity->sizePolicy().hasHeightForWidth());
        pedtCity->setSizePolicy(sizePolicy1);
        pedtCity->setMinimumSize(QSize(150, 0));

        verticalLayout_9->addWidget(pedtCity);

        pcbxSkill = new QComboBox(grbMusicien);
        pcbxSkill->setObjectName("pcbxSkill");
        pcbxSkill->setMinimumSize(QSize(150, 0));

        verticalLayout_9->addWidget(pcbxSkill);


        horizontalLayout_9->addLayout(verticalLayout_9);


        verticalLayout_10->addLayout(horizontalLayout_9);


        verticalLayout_12->addWidget(grbMusicien);

        verticalSpacer_7 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_12->addItem(verticalSpacer_7);

        gbxUserInterface = new QGroupBox(tab_0_User);
        gbxUserInterface->setObjectName("gbxUserInterface");
        verticalLayout_11 = new QVBoxLayout(gbxUserInterface);
        verticalLayout_11->setObjectName("verticalLayout_11");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setObjectName("verticalLayout_5");
        lblSkin = new QLabel(gbxUserInterface);
        lblSkin->setObjectName("lblSkin");

        verticalLayout_5->addWidget(lblSkin);

        lblMeterStyle = new QLabel(gbxUserInterface);
        lblMeterStyle->setObjectName("lblMeterStyle");

        verticalLayout_5->addWidget(lblMeterStyle);

        lblLanguage = new QLabel(gbxUserInterface);
        lblLanguage->setObjectName("lblLanguage");

        verticalLayout_5->addWidget(lblLanguage);

        lblMixerRows = new QLabel(gbxUserInterface);
        lblMixerRows->setObjectName("lblMixerRows");

        verticalLayout_5->addWidget(lblMixerRows);

        lblAudioAlerts = new QLabel(gbxUserInterface);
        lblAudioAlerts->setObjectName("lblAudioAlerts");

        verticalLayout_5->addWidget(lblAudioAlerts);


        horizontalLayout_3->addLayout(verticalLayout_5);

        verticalLayout_7 = new QVBoxLayout();
        verticalLayout_7->setObjectName("verticalLayout_7");
        cbxSkin = new QComboBox(gbxUserInterface);
        cbxSkin->setObjectName("cbxSkin");

        verticalLayout_7->addWidget(cbxSkin);

        cbxMeterStyle = new QComboBox(gbxUserInterface);
        cbxMeterStyle->setObjectName("cbxMeterStyle");

        verticalLayout_7->addWidget(cbxMeterStyle);

        cbxLanguage = new CLanguageComboBox(gbxUserInterface);
        cbxLanguage->setObjectName("cbxLanguage");
        cbxLanguage->setMinimumSize(QSize(150, 0));

        verticalLayout_7->addWidget(cbxLanguage);

        spnMixerRows = new QSpinBox(gbxUserInterface);
        spnMixerRows->setObjectName("spnMixerRows");
        sizePolicy1.setHeightForWidth(spnMixerRows->sizePolicy().hasHeightForWidth());
        spnMixerRows->setSizePolicy(sizePolicy1);
        spnMixerRows->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        spnMixerRows->setMinimum(1);
        spnMixerRows->setMaximum(8);

        verticalLayout_7->addWidget(spnMixerRows);

        chbAudioAlerts = new QCheckBox(gbxUserInterface);
        chbAudioAlerts->setObjectName("chbAudioAlerts");

        verticalLayout_7->addWidget(chbAudioAlerts);


        horizontalLayout_3->addLayout(verticalLayout_7);


        verticalLayout_11->addLayout(horizontalLayout_3);


        verticalLayout_12->addWidget(gbxUserInterface);

        verticalSpacer_9 = new QSpacerItem(20, 14, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_12->addItem(verticalSpacer_9);


        horizontalLayout_10->addLayout(verticalLayout_12);

        horizontalSpacer_2 = new QSpacerItem(0, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer_2);


        verticalLayout_16->addLayout(horizontalLayout_10);

        tabSettings->addTab(tab_0_User, QString());
        tab_1_AudioNet = new QWidget();
        tab_1_AudioNet->setObjectName("tab_1_AudioNet");
        horizontalLayout_6 = new QHBoxLayout(tab_1_AudioNet);
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName("verticalLayout_3");
        lblSoundcardDevice = new QLabel(tab_1_AudioNet);
        lblSoundcardDevice->setObjectName("lblSoundcardDevice");

        verticalLayout_3->addWidget(lblSoundcardDevice);

        cbxSoundcard = new QComboBox(tab_1_AudioNet);
        cbxSoundcard->setObjectName("cbxSoundcard");
        sizePolicy1.setHeightForWidth(cbxSoundcard->sizePolicy().hasHeightForWidth());
        cbxSoundcard->setSizePolicy(sizePolicy1);
        cbxSoundcard->setMinimumSize(QSize(0, 0));

        verticalLayout_3->addWidget(cbxSoundcard);

        verticalSpacer_3 = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_3);

        butDriverSetup = new QPushButton(tab_1_AudioNet);
        butDriverSetup->setObjectName("butDriverSetup");
        sizePolicy1.setHeightForWidth(butDriverSetup->sizePolicy().hasHeightForWidth());
        butDriverSetup->setSizePolicy(sizePolicy1);
        butDriverSetup->setAutoDefault(false);

        verticalLayout_3->addWidget(butDriverSetup);

        verticalSpacer_5 = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_5);

        FrameSoundcardChannelSelection = new QFrame(tab_1_AudioNet);
        FrameSoundcardChannelSelection->setObjectName("FrameSoundcardChannelSelection");
        sizePolicy.setHeightForWidth(FrameSoundcardChannelSelection->sizePolicy().hasHeightForWidth());
        FrameSoundcardChannelSelection->setSizePolicy(sizePolicy);
        FrameSoundcardChannelSelection->setFrameShape(QFrame::NoFrame);
        FrameSoundcardChannelSelection->setFrameShadow(QFrame::Plain);
        _2 = new QVBoxLayout(FrameSoundcardChannelSelection);
        _2->setObjectName("_2");
        _2->setContentsMargins(0, 0, 0, 0);
        lblInChannelMapping = new QLabel(FrameSoundcardChannelSelection);
        lblInChannelMapping->setObjectName("lblInChannelMapping");

        _2->addWidget(lblInChannelMapping);

        _3 = new QHBoxLayout();
        _3->setSpacing(3);
        _3->setObjectName("_3");
        _4 = new QVBoxLayout();
        _4->setObjectName("_4");
        lblLInChan = new QLabel(FrameSoundcardChannelSelection);
        lblLInChan->setObjectName("lblLInChan");
        sizePolicy.setHeightForWidth(lblLInChan->sizePolicy().hasHeightForWidth());
        lblLInChan->setSizePolicy(sizePolicy);

        _4->addWidget(lblLInChan);

        lblRInChan = new QLabel(FrameSoundcardChannelSelection);
        lblRInChan->setObjectName("lblRInChan");
        sizePolicy.setHeightForWidth(lblRInChan->sizePolicy().hasHeightForWidth());
        lblRInChan->setSizePolicy(sizePolicy);

        _4->addWidget(lblRInChan);


        _3->addLayout(_4);

        _5 = new QVBoxLayout();
        _5->setSpacing(3);
        _5->setObjectName("_5");
        cbxLInChan = new QComboBox(FrameSoundcardChannelSelection);
        cbxLInChan->setObjectName("cbxLInChan");

        _5->addWidget(cbxLInChan);

        cbxRInChan = new QComboBox(FrameSoundcardChannelSelection);
        cbxRInChan->setObjectName("cbxRInChan");

        _5->addWidget(cbxRInChan);


        _3->addLayout(_5);


        _2->addLayout(_3);

        lblOutChannelMapping = new QLabel(FrameSoundcardChannelSelection);
        lblOutChannelMapping->setObjectName("lblOutChannelMapping");

        _2->addWidget(lblOutChannelMapping);

        _6 = new QHBoxLayout();
        _6->setSpacing(3);
        _6->setObjectName("_6");
        _7 = new QVBoxLayout();
        _7->setObjectName("_7");
        lblLOutChan = new QLabel(FrameSoundcardChannelSelection);
        lblLOutChan->setObjectName("lblLOutChan");
        sizePolicy.setHeightForWidth(lblLOutChan->sizePolicy().hasHeightForWidth());
        lblLOutChan->setSizePolicy(sizePolicy);

        _7->addWidget(lblLOutChan);

        lblROutChan = new QLabel(FrameSoundcardChannelSelection);
        lblROutChan->setObjectName("lblROutChan");
        sizePolicy.setHeightForWidth(lblROutChan->sizePolicy().hasHeightForWidth());
        lblROutChan->setSizePolicy(sizePolicy);

        _7->addWidget(lblROutChan);


        _6->addLayout(_7);

        _8 = new QVBoxLayout();
        _8->setSpacing(3);
        _8->setObjectName("_8");
        cbxLOutChan = new QComboBox(FrameSoundcardChannelSelection);
        cbxLOutChan->setObjectName("cbxLOutChan");

        _8->addWidget(cbxLOutChan);

        cbxROutChan = new QComboBox(FrameSoundcardChannelSelection);
        cbxROutChan->setObjectName("cbxROutChan");

        _8->addWidget(cbxROutChan);


        _6->addLayout(_8);


        _2->addLayout(_6);


        verticalLayout_3->addWidget(FrameSoundcardChannelSelection);

        verticalSpacer_6 = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_6);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        lblAudioChannels = new QLabel(tab_1_AudioNet);
        lblAudioChannels->setObjectName("lblAudioChannels");

        verticalLayout_2->addWidget(lblAudioChannels);

        lblAudioQuality = new QLabel(tab_1_AudioNet);
        lblAudioQuality->setObjectName("lblAudioQuality");

        verticalLayout_2->addWidget(lblAudioQuality);


        horizontalLayout->addLayout(verticalLayout_2);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        cbxAudioChannels = new QComboBox(tab_1_AudioNet);
        cbxAudioChannels->setObjectName("cbxAudioChannels");

        verticalLayout->addWidget(cbxAudioChannels);

        cbxAudioQuality = new QComboBox(tab_1_AudioNet);
        cbxAudioQuality->setObjectName("cbxAudioQuality");

        verticalLayout->addWidget(cbxAudioQuality);


        horizontalLayout->addLayout(verticalLayout);


        verticalLayout_3->addLayout(horizontalLayout);

        verticalSpacer_10 = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_10);

        grbSoundCrdBufDelay = new QGroupBox(tab_1_AudioNet);
        grbSoundCrdBufDelay->setObjectName("grbSoundCrdBufDelay");
        _9 = new QVBoxLayout(grbSoundCrdBufDelay);
        _9->setObjectName("_9");
        rbtBufferDelayPreferred = new QRadioButton(grbSoundCrdBufDelay);
        rbtBufferDelayPreferred->setObjectName("rbtBufferDelayPreferred");
        sizePolicy1.setHeightForWidth(rbtBufferDelayPreferred->sizePolicy().hasHeightForWidth());
        rbtBufferDelayPreferred->setSizePolicy(sizePolicy1);

        _9->addWidget(rbtBufferDelayPreferred);

        rbtBufferDelayDefault = new QRadioButton(grbSoundCrdBufDelay);
        rbtBufferDelayDefault->setObjectName("rbtBufferDelayDefault");
        sizePolicy1.setHeightForWidth(rbtBufferDelayDefault->sizePolicy().hasHeightForWidth());
        rbtBufferDelayDefault->setSizePolicy(sizePolicy1);

        _9->addWidget(rbtBufferDelayDefault);

        rbtBufferDelaySafe = new QRadioButton(grbSoundCrdBufDelay);
        rbtBufferDelaySafe->setObjectName("rbtBufferDelaySafe");
        sizePolicy1.setHeightForWidth(rbtBufferDelaySafe->sizePolicy().hasHeightForWidth());
        rbtBufferDelaySafe->setSizePolicy(sizePolicy1);

        _9->addWidget(rbtBufferDelaySafe);


        verticalLayout_3->addWidget(grbSoundCrdBufDelay);


        horizontalLayout_5->addLayout(verticalLayout_3);

        horizontalSpacer_3 = new QSpacerItem(0, 20, QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_3);

        verticalLayout_13 = new QVBoxLayout();
        verticalLayout_13->setObjectName("verticalLayout_13");
        grbJitterBuffer = new QGroupBox(tab_1_AudioNet);
        grbJitterBuffer->setObjectName("grbJitterBuffer");
        _15 = new QVBoxLayout(grbJitterBuffer);
        _15->setObjectName("_15");
        chbAutoJitBuf = new QCheckBox(grbJitterBuffer);
        chbAutoJitBuf->setObjectName("chbAutoJitBuf");
        sizePolicy1.setHeightForWidth(chbAutoJitBuf->sizePolicy().hasHeightForWidth());
        chbAutoJitBuf->setSizePolicy(sizePolicy1);

        _15->addWidget(chbAutoJitBuf);

        _16 = new QHBoxLayout();
        _16->setObjectName("_16");
        lblNetBufLabel = new QLabel(grbJitterBuffer);
        lblNetBufLabel->setObjectName("lblNetBufLabel");
        lblNetBufLabel->setAlignment(Qt::AlignCenter);
        lblNetBufLabel->setWordWrap(false);

        _16->addWidget(lblNetBufLabel);

        lblNetBufServerLabel = new QLabel(grbJitterBuffer);
        lblNetBufServerLabel->setObjectName("lblNetBufServerLabel");
        lblNetBufServerLabel->setAlignment(Qt::AlignCenter);
        lblNetBufServerLabel->setWordWrap(false);

        _16->addWidget(lblNetBufServerLabel);


        _15->addLayout(_16);

        _17 = new QHBoxLayout();
        _17->setObjectName("_17");
        lblNetBuf = new QLabel(grbJitterBuffer);
        lblNetBuf->setObjectName("lblNetBuf");
        lblNetBuf->setAlignment(Qt::AlignCenter);
        lblNetBuf->setWordWrap(false);

        _17->addWidget(lblNetBuf);

        lblNetBufServer = new QLabel(grbJitterBuffer);
        lblNetBufServer->setObjectName("lblNetBufServer");
        lblNetBufServer->setAlignment(Qt::AlignCenter);
        lblNetBufServer->setWordWrap(false);

        _17->addWidget(lblNetBufServer);


        _15->addLayout(_17);

        _18 = new QHBoxLayout();
        _18->setObjectName("_18");
        spacerItem = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        _18->addItem(spacerItem);

        sldNetBuf = new QSlider(grbJitterBuffer);
        sldNetBuf->setObjectName("sldNetBuf");
        sldNetBuf->setMinimum(1);
        sldNetBuf->setMaximum(20);
        sldNetBuf->setPageStep(1);
        sldNetBuf->setOrientation(Qt::Vertical);
        sldNetBuf->setTickPosition(QSlider::TicksBothSides);

        _18->addWidget(sldNetBuf);

        spacerItem1 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        _18->addItem(spacerItem1);

        sldNetBufServer = new QSlider(grbJitterBuffer);
        sldNetBufServer->setObjectName("sldNetBufServer");
        sldNetBufServer->setMinimum(1);
        sldNetBufServer->setMaximum(20);
        sldNetBufServer->setPageStep(1);
        sldNetBufServer->setOrientation(Qt::Vertical);
        sldNetBufServer->setTickPosition(QSlider::TicksBothSides);

        _18->addWidget(sldNetBufServer);


        _15->addLayout(_18);


        verticalLayout_13->addWidget(grbJitterBuffer);

        chbSmallNetworkBuffers = new QCheckBox(tab_1_AudioNet);
        chbSmallNetworkBuffers->setObjectName("chbSmallNetworkBuffers");
        sizePolicy1.setHeightForWidth(chbSmallNetworkBuffers->sizePolicy().hasHeightForWidth());
        chbSmallNetworkBuffers->setSizePolicy(sizePolicy1);

        verticalLayout_13->addWidget(chbSmallNetworkBuffers);

        grbUpstreamValue = new QGroupBox(tab_1_AudioNet);
        grbUpstreamValue->setObjectName("grbUpstreamValue");
        verticalLayout_14 = new QVBoxLayout(grbUpstreamValue);
        verticalLayout_14->setSpacing(2);
        verticalLayout_14->setObjectName("verticalLayout_14");
        _10 = new QGridLayout();
        _10->setObjectName("_10");
        horizontalSpacer_9 = new QSpacerItem(1, 5, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _10->addItem(horizontalSpacer_9, 0, 4, 1, 1);

        lblUpstreamUnit = new QLabel(grbUpstreamValue);
        lblUpstreamUnit->setObjectName("lblUpstreamUnit");
        lblUpstreamUnit->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        _10->addWidget(lblUpstreamUnit, 0, 3, 1, 1);

        lblUpstreamValue = new QLabel(grbUpstreamValue);
        lblUpstreamValue->setObjectName("lblUpstreamValue");
        lblUpstreamValue->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        lblUpstreamValue->setMargin(2);

        _10->addWidget(lblUpstreamValue, 0, 1, 1, 1);

        horizontalSpacer_10 = new QSpacerItem(1, 5, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _10->addItem(horizontalSpacer_10, 0, 0, 1, 1);


        verticalLayout_14->addLayout(_10);


        verticalLayout_13->addWidget(grbUpstreamValue);


        horizontalLayout_5->addLayout(verticalLayout_13);

        horizontalSpacer_5 = new QSpacerItem(0, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_5);


        horizontalLayout_6->addLayout(horizontalLayout_5);

        tabSettings->addTab(tab_1_AudioNet, QString());
        tab_2_advanced = new QWidget();
        tab_2_advanced->setObjectName("tab_2_advanced");
        verticalLayout_15 = new QVBoxLayout(tab_2_advanced);
        verticalLayout_15->setObjectName("verticalLayout_15");
        verticalLayout_6 = new QVBoxLayout();
        verticalLayout_6->setObjectName("verticalLayout_6");
        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        horizontalSpacer_6 = new QSpacerItem(10, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_6);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalSpacer_2 = new QSpacerItem(17, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        verticalLayout_4->addItem(verticalSpacer_2);

        lblCustomDirectories = new QLabel(tab_2_advanced);
        lblCustomDirectories->setObjectName("lblCustomDirectories");

        verticalLayout_4->addWidget(lblCustomDirectories);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        cbxCustomDirectories = new QComboBox(tab_2_advanced);
        cbxCustomDirectories->setObjectName("cbxCustomDirectories");
        cbxCustomDirectories->setEditable(true);

        hboxLayout->addWidget(cbxCustomDirectories);

        tbtDeleteCustomDirectory = new QToolButton(tab_2_advanced);
        tbtDeleteCustomDirectory->setObjectName("tbtDeleteCustomDirectory");

        hboxLayout->addWidget(tbtDeleteCustomDirectory);


        verticalLayout_4->addLayout(hboxLayout);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_4);

        lblNewClientLevel = new QLabel(tab_2_advanced);
        lblNewClientLevel->setObjectName("lblNewClientLevel");

        verticalLayout_4->addWidget(lblNewClientLevel);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        edtNewClientLevel = new QLineEdit(tab_2_advanced);
        edtNewClientLevel->setObjectName("edtNewClientLevel");

        horizontalLayout_4->addWidget(edtNewClientLevel);

        lblPercentUnit = new QLabel(tab_2_advanced);
        lblPercentUnit->setObjectName("lblPercentUnit");

        horizontalLayout_4->addWidget(lblPercentUnit);


        verticalLayout_4->addLayout(horizontalLayout_4);

        verticalSpacer_11 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_11);

        lblInputBoost = new QLabel(tab_2_advanced);
        lblInputBoost->setObjectName("lblInputBoost");

        verticalLayout_4->addWidget(lblInputBoost);

        cbxInputBoost = new QComboBox(tab_2_advanced);
        cbxInputBoost->setObjectName("cbxInputBoost");

        verticalLayout_4->addWidget(cbxInputBoost);

        verticalSpacer_13 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_13);

        chbDetectFeedback = new QCheckBox(tab_2_advanced);
        chbDetectFeedback->setObjectName("chbDetectFeedback");
        sizePolicy1.setHeightForWidth(chbDetectFeedback->sizePolicy().hasHeightForWidth());
        chbDetectFeedback->setSizePolicy(sizePolicy1);

        verticalLayout_4->addWidget(chbDetectFeedback);

        verticalSpacer_15 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_15);


        horizontalLayout_8->addLayout(verticalLayout_4);

        horizontalSpacer_4 = new QSpacerItem(10, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_4);


        verticalLayout_6->addLayout(horizontalLayout_8);

        horizontalLayout_12 = new QHBoxLayout();
        horizontalLayout_12->setObjectName("horizontalLayout_12");
        horizontalSpacer_7 = new QSpacerItem(20, 20, QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);

        horizontalLayout_12->addItem(horizontalSpacer_7);

        groupBox = new QGroupBox(tab_2_advanced);
        groupBox->setObjectName("groupBox");
        horizontalLayout_2 = new QHBoxLayout(groupBox);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        _19 = new QVBoxLayout();
        _19->setSpacing(3);
        _19->setObjectName("_19");
        lblAudioPan = new QLabel(groupBox);
        lblAudioPan->setObjectName("lblAudioPan");
        lblAudioPan->setAlignment(Qt::AlignCenter);

        _19->addWidget(lblAudioPan);

        _20 = new QHBoxLayout();
        _20->setObjectName("_20");
        spacerItem2 = new QSpacerItem(0, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        _20->addItem(spacerItem2);

        sldAudioPan = new QSlider(groupBox);
        sldAudioPan->setObjectName("sldAudioPan");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(sldAudioPan->sizePolicy().hasHeightForWidth());
        sldAudioPan->setSizePolicy(sizePolicy2);
        sldAudioPan->setMinimumSize(QSize(0, 0));
        sldAudioPan->setPageStep(1);
        sldAudioPan->setOrientation(Qt::Horizontal);
        sldAudioPan->setTickPosition(QSlider::TicksBothSides);

        _20->addWidget(sldAudioPan);

        spacerItem3 = new QSpacerItem(0, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        _20->addItem(spacerItem3);


        _19->addLayout(_20);

        lblAudioPanValue = new QLabel(groupBox);
        lblAudioPanValue->setObjectName("lblAudioPanValue");
        lblAudioPanValue->setMinimumSize(QSize(50, 0));
        lblAudioPanValue->setAlignment(Qt::AlignCenter);
        lblAudioPanValue->setWordWrap(false);

        _19->addWidget(lblAudioPanValue);


        horizontalLayout_2->addLayout(_19);


        horizontalLayout_12->addWidget(groupBox);

        horizontalSpacer_8 = new QSpacerItem(20, 20, QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);

        horizontalLayout_12->addItem(horizontalSpacer_8);


        verticalLayout_6->addLayout(horizontalLayout_12);


        verticalLayout_15->addLayout(verticalLayout_6);

        verticalSpacer = new QSpacerItem(367, 13, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_15->addItem(verticalSpacer);

        tabSettings->addTab(tab_2_advanced, QString());
        tab_3_midi = new QWidget();
        tab_3_midi->setObjectName("tab_3_midi");
        verticalLayout_18 = new QVBoxLayout(tab_3_midi);
        verticalLayout_18->setObjectName("verticalLayout_18");
        wdgMidiDeviceContainer = new QWidget(tab_3_midi);
        wdgMidiDeviceContainer->setObjectName("wdgMidiDeviceContainer");
        horizontalLayout_midiDevice = new QHBoxLayout(wdgMidiDeviceContainer);
        horizontalLayout_midiDevice->setObjectName("horizontalLayout_midiDevice");
        horizontalLayout_midiDevice->setContentsMargins(0, 0, 0, 0);

        verticalLayout_18->addWidget(wdgMidiDeviceContainer);

        grbMidiControls = new QGroupBox(tab_3_midi);
        grbMidiControls->setObjectName("grbMidiControls");
        grbMidiControls->setCheckable(true);
        grbMidiControls->setChecked(false);
        gridLayout = new QGridLayout(grbMidiControls);
        gridLayout->setObjectName("gridLayout");
        horizontalLayout_17 = new QHBoxLayout();
        horizontalLayout_17->setObjectName("horizontalLayout_17");
        lblChannel = new QLabel(grbMidiControls);
        lblChannel->setObjectName("lblChannel");
        lblChannel->setMaximumSize(QSize(200, 16777215));
        QFont font;
        font.setBold(false);
        lblChannel->setFont(font);

        horizontalLayout_17->addWidget(lblChannel);

        cbxChannel = new QComboBox(grbMidiControls);
        cbxChannel->addItem(QString());
        cbxChannel->addItem(QString());
        cbxChannel->addItem(QString());
        cbxChannel->addItem(QString());
        cbxChannel->addItem(QString());
        cbxChannel->addItem(QString());
        cbxChannel->addItem(QString());
        cbxChannel->addItem(QString());
        cbxChannel->addItem(QString());
        cbxChannel->addItem(QString());
        cbxChannel->addItem(QString());
        cbxChannel->addItem(QString());
        cbxChannel->addItem(QString());
        cbxChannel->addItem(QString());
        cbxChannel->addItem(QString());
        cbxChannel->addItem(QString());
        cbxChannel->addItem(QString());
        cbxChannel->setObjectName("cbxChannel");
        cbxChannel->setMinimumSize(QSize(80, 0));
        cbxChannel->setMaximumSize(QSize(80, 16777215));
        cbxChannel->setFont(font);

        horizontalLayout_17->addWidget(cbxChannel);

        horizontalSpacer_15 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_17->addItem(horizontalSpacer_15);

        chbMIDIPickupMode = new QCheckBox(grbMidiControls);
        chbMIDIPickupMode->setObjectName("chbMIDIPickupMode");

        horizontalLayout_17->addWidget(chbMIDIPickupMode);

        horizontalSpacer_18 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_17->addItem(horizontalSpacer_18);


        gridLayout->addLayout(horizontalLayout_17, 2, 0, 1, 1);

        grbMidiMuteMyself = new QGroupBox(grbMidiControls);
        grbMidiMuteMyself->setObjectName("grbMidiMuteMyself");
        grbMidiMuteMyself->setFont(font);
        grbMidiMuteMyself->setCheckable(true);
        horizontalLayout_20 = new QHBoxLayout(grbMidiMuteMyself);
        horizontalLayout_20->setObjectName("horizontalLayout_20");
        horizontalLayout_16 = new QHBoxLayout();
        horizontalLayout_16->setObjectName("horizontalLayout_16");
        lblMuteMyselfCC = new QLabel(grbMidiMuteMyself);
        lblMuteMyselfCC->setObjectName("lblMuteMyselfCC");
        lblMuteMyselfCC->setMinimumSize(QSize(105, 0));
        lblMuteMyselfCC->setFont(font);

        horizontalLayout_16->addWidget(lblMuteMyselfCC);

        spnMuteMyself = new QSpinBox(grbMidiMuteMyself);
        spnMuteMyself->setObjectName("spnMuteMyself");
        spnMuteMyself->setMinimumSize(QSize(85, 0));
        spnMuteMyself->setFont(font);
        spnMuteMyself->setMinimum(0);
        spnMuteMyself->setMaximum(127);

        horizontalLayout_16->addWidget(spnMuteMyself);

        butLearnMuteMyself = new QPushButton(grbMidiMuteMyself);
        butLearnMuteMyself->setObjectName("butLearnMuteMyself");
        butLearnMuteMyself->setFont(font);
        butLearnMuteMyself->setAutoDefault(false);

        horizontalLayout_16->addWidget(butLearnMuteMyself);

        horizontalSpacer_17 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_16->addItem(horizontalSpacer_17);


        horizontalLayout_20->addLayout(horizontalLayout_16);


        gridLayout->addWidget(grbMidiMuteMyself, 4, 0, 1, 1);

        verticalSpacer_18 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer_18, 3, 0, 1, 1);

        verticalSpacer_17 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer_17, 5, 0, 1, 1);

        gridLayoutMain = new QGridLayout();
        gridLayoutMain->setObjectName("gridLayoutMain");
        verticalSpacer_14 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayoutMain->addItem(verticalSpacer_14, 3, 0, 1, 1);

        grbMidiFader = new QGroupBox(grbMidiControls);
        grbMidiFader->setObjectName("grbMidiFader");
        grbMidiFader->setFont(font);
        grbMidiFader->setCheckable(true);
        horizontalLayout_19 = new QHBoxLayout(grbMidiFader);
        horizontalLayout_19->setObjectName("horizontalLayout_19");
        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setObjectName("horizontalLayout_11");
        lblFaderOffset = new QLabel(grbMidiFader);
        lblFaderOffset->setObjectName("lblFaderOffset");
        lblFaderOffset->setMinimumSize(QSize(105, 0));
        lblFaderOffset->setFont(font);

        horizontalLayout_11->addWidget(lblFaderOffset);

        spnFaderOffset = new QSpinBox(grbMidiFader);
        spnFaderOffset->setObjectName("spnFaderOffset");
        spnFaderOffset->setMinimumSize(QSize(85, 0));
        spnFaderOffset->setFont(font);
        spnFaderOffset->setMaximum(127);

        horizontalLayout_11->addWidget(spnFaderOffset);

        butLearnFaderOffset = new QPushButton(grbMidiFader);
        butLearnFaderOffset->setObjectName("butLearnFaderOffset");
        butLearnFaderOffset->setFont(font);
        butLearnFaderOffset->setAutoDefault(false);

        horizontalLayout_11->addWidget(butLearnFaderOffset);

        horizontalSpacer_11 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_11);

        lblFaderCount = new QLabel(grbMidiFader);
        lblFaderCount->setObjectName("lblFaderCount");
        lblFaderCount->setFont(font);

        horizontalLayout_11->addWidget(lblFaderCount);

        spnFaderCount = new QSpinBox(grbMidiFader);
        spnFaderCount->setObjectName("spnFaderCount");
        spnFaderCount->setMinimumSize(QSize(85, 0));
        spnFaderCount->setFont(font);
        spnFaderCount->setMinimum(1);
        spnFaderCount->setMaximum(127);

        horizontalLayout_11->addWidget(spnFaderCount);


        horizontalLayout_19->addLayout(horizontalLayout_11);


        gridLayoutMain->addWidget(grbMidiFader, 0, 0, 1, 1);

        grbMidiMute = new QGroupBox(grbMidiControls);
        grbMidiMute->setObjectName("grbMidiMute");
        grbMidiMute->setFont(font);
        grbMidiMute->setCheckable(true);
        horizontalLayout_18 = new QHBoxLayout(grbMidiMute);
        horizontalLayout_18->setObjectName("horizontalLayout_18");
        horizontalLayout_15 = new QHBoxLayout();
        horizontalLayout_15->setObjectName("horizontalLayout_15");
        lblMuteOffset = new QLabel(grbMidiMute);
        lblMuteOffset->setObjectName("lblMuteOffset");
        lblMuteOffset->setMinimumSize(QSize(105, 0));
        lblMuteOffset->setFont(font);

        horizontalLayout_15->addWidget(lblMuteOffset);

        spnMuteOffset = new QSpinBox(grbMidiMute);
        spnMuteOffset->setObjectName("spnMuteOffset");
        spnMuteOffset->setMinimumSize(QSize(85, 0));
        spnMuteOffset->setFont(font);
        spnMuteOffset->setMaximum(127);

        horizontalLayout_15->addWidget(spnMuteOffset);

        butLearnMuteOffset = new QPushButton(grbMidiMute);
        butLearnMuteOffset->setObjectName("butLearnMuteOffset");
        butLearnMuteOffset->setFont(font);
        butLearnMuteOffset->setAutoDefault(false);

        horizontalLayout_15->addWidget(butLearnMuteOffset);

        horizontalSpacer_14 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_15->addItem(horizontalSpacer_14);

        lblMuteCount = new QLabel(grbMidiMute);
        lblMuteCount->setObjectName("lblMuteCount");
        lblMuteCount->setFont(font);

        horizontalLayout_15->addWidget(lblMuteCount);

        spnMuteCount = new QSpinBox(grbMidiMute);
        spnMuteCount->setObjectName("spnMuteCount");
        spnMuteCount->setMinimumSize(QSize(85, 0));
        spnMuteCount->setFont(font);
        spnMuteCount->setMinimum(1);
        spnMuteCount->setMaximum(127);

        horizontalLayout_15->addWidget(spnMuteCount);


        horizontalLayout_18->addLayout(horizontalLayout_15);


        gridLayoutMain->addWidget(grbMidiMute, 6, 0, 1, 1);

        verticalSpacer_12 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayoutMain->addItem(verticalSpacer_12, 1, 0, 1, 1);

        grbMidiSolo = new QGroupBox(grbMidiControls);
        grbMidiSolo->setObjectName("grbMidiSolo");
        grbMidiSolo->setFont(font);
        grbMidiSolo->setCheckable(true);
        horizontalLayout_21 = new QHBoxLayout(grbMidiSolo);
        horizontalLayout_21->setObjectName("horizontalLayout_21");
        horizontalLayout_14 = new QHBoxLayout();
        horizontalLayout_14->setObjectName("horizontalLayout_14");
        lblSoloOffset = new QLabel(grbMidiSolo);
        lblSoloOffset->setObjectName("lblSoloOffset");
        lblSoloOffset->setMinimumSize(QSize(105, 0));
        lblSoloOffset->setFont(font);

        horizontalLayout_14->addWidget(lblSoloOffset);

        spnSoloOffset = new QSpinBox(grbMidiSolo);
        spnSoloOffset->setObjectName("spnSoloOffset");
        spnSoloOffset->setMinimumSize(QSize(85, 0));
        spnSoloOffset->setFont(font);
        spnSoloOffset->setMaximum(127);

        horizontalLayout_14->addWidget(spnSoloOffset);

        butLearnSoloOffset = new QPushButton(grbMidiSolo);
        butLearnSoloOffset->setObjectName("butLearnSoloOffset");
        butLearnSoloOffset->setFont(font);
        butLearnSoloOffset->setAutoDefault(false);

        horizontalLayout_14->addWidget(butLearnSoloOffset);

        horizontalSpacer_13 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_14->addItem(horizontalSpacer_13);

        lblSoloCount = new QLabel(grbMidiSolo);
        lblSoloCount->setObjectName("lblSoloCount");
        lblSoloCount->setFont(font);

        horizontalLayout_14->addWidget(lblSoloCount);

        spnSoloCount = new QSpinBox(grbMidiSolo);
        spnSoloCount->setObjectName("spnSoloCount");
        spnSoloCount->setMinimumSize(QSize(85, 0));
        spnSoloCount->setFont(font);
        spnSoloCount->setMinimum(1);
        spnSoloCount->setMaximum(127);

        horizontalLayout_14->addWidget(spnSoloCount);


        horizontalLayout_21->addLayout(horizontalLayout_14);


        gridLayoutMain->addWidget(grbMidiSolo, 4, 0, 1, 1);

        grbMidiPan = new QGroupBox(grbMidiControls);
        grbMidiPan->setObjectName("grbMidiPan");
        grbMidiPan->setFont(font);
        grbMidiPan->setCheckable(true);
        horizontalLayout_22 = new QHBoxLayout(grbMidiPan);
        horizontalLayout_22->setObjectName("horizontalLayout_22");
        horizontalLayout_13 = new QHBoxLayout();
        horizontalLayout_13->setObjectName("horizontalLayout_13");
        lblPanOffset = new QLabel(grbMidiPan);
        lblPanOffset->setObjectName("lblPanOffset");
        lblPanOffset->setMinimumSize(QSize(105, 0));
        lblPanOffset->setFont(font);

        horizontalLayout_13->addWidget(lblPanOffset);

        spnPanOffset = new QSpinBox(grbMidiPan);
        spnPanOffset->setObjectName("spnPanOffset");
        spnPanOffset->setMinimumSize(QSize(85, 0));
        spnPanOffset->setFont(font);
        spnPanOffset->setMaximum(127);

        horizontalLayout_13->addWidget(spnPanOffset);

        butLearnPanOffset = new QPushButton(grbMidiPan);
        butLearnPanOffset->setObjectName("butLearnPanOffset");
        butLearnPanOffset->setFont(font);
        butLearnPanOffset->setAutoDefault(false);

        horizontalLayout_13->addWidget(butLearnPanOffset);

        horizontalSpacer_12 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_13->addItem(horizontalSpacer_12);

        lblPanCount = new QLabel(grbMidiPan);
        lblPanCount->setObjectName("lblPanCount");
        lblPanCount->setFont(font);

        horizontalLayout_13->addWidget(lblPanCount);

        spnPanCount = new QSpinBox(grbMidiPan);
        spnPanCount->setObjectName("spnPanCount");
        spnPanCount->setMinimumSize(QSize(85, 0));
        spnPanCount->setFont(font);
        spnPanCount->setMinimum(1);
        spnPanCount->setMaximum(127);

        horizontalLayout_13->addWidget(spnPanCount);


        horizontalLayout_22->addLayout(horizontalLayout_13);


        gridLayoutMain->addWidget(grbMidiPan, 2, 0, 1, 1);

        verticalSpacer_16 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayoutMain->addItem(verticalSpacer_16, 5, 0, 1, 1);


        gridLayout->addLayout(gridLayoutMain, 6, 0, 1, 1);

        horizontalLayout_23 = new QHBoxLayout();
        horizontalLayout_23->setObjectName("horizontalLayout_23");
        lblMidiDevice = new QLabel(grbMidiControls);
        lblMidiDevice->setObjectName("lblMidiDevice");

        horizontalLayout_23->addWidget(lblMidiDevice);

        cbxMidiDevice = new QComboBox(grbMidiControls);
        cbxMidiDevice->setObjectName("cbxMidiDevice");
        cbxMidiDevice->setMinimumSize(QSize(350, 0));

        horizontalLayout_23->addWidget(cbxMidiDevice);

        horizontalSpacer_16 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_23->addItem(horizontalSpacer_16);


        gridLayout->addLayout(horizontalLayout_23, 0, 0, 1, 1);

        verticalSpacer_19 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer_19, 1, 0, 1, 1);


        verticalLayout_18->addWidget(grbMidiControls);

        tabSettings->addTab(tab_3_midi, QString());

        horizontalLayout_7->addWidget(tabSettings);

#if QT_CONFIG(shortcut)
        plblAlias->setBuddy(pedtAlias);
        plblInstrument->setBuddy(pcbxInstrument);
        plblCountry->setBuddy(pcbxCountry);
        plblCity->setBuddy(pedtCity);
        plblSkill->setBuddy(pcbxSkill);
        lblSkin->setBuddy(cbxSkin);
        lblMeterStyle->setBuddy(cbxMeterStyle);
        lblLanguage->setBuddy(cbxLanguage);
        lblMixerRows->setBuddy(spnMixerRows);
        lblSoundcardDevice->setBuddy(cbxSoundcard);
        lblLInChan->setBuddy(cbxLInChan);
        lblRInChan->setBuddy(cbxRInChan);
        lblLOutChan->setBuddy(cbxLOutChan);
        lblROutChan->setBuddy(cbxROutChan);
        lblAudioChannels->setBuddy(cbxAudioChannels);
        lblAudioQuality->setBuddy(cbxAudioQuality);
        lblCustomDirectories->setBuddy(cbxCustomDirectories);
        lblNewClientLevel->setBuddy(edtNewClientLevel);
        lblInputBoost->setBuddy(cbxInputBoost);
        lblAudioPan->setBuddy(sldAudioPan);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(tabSettings, pedtAlias);
        QWidget::setTabOrder(pedtAlias, pcbxInstrument);
        QWidget::setTabOrder(pcbxInstrument, pcbxCountry);
        QWidget::setTabOrder(pcbxCountry, pedtCity);
        QWidget::setTabOrder(pedtCity, pcbxSkill);
        QWidget::setTabOrder(pcbxSkill, cbxSkin);
        QWidget::setTabOrder(cbxSkin, cbxMeterStyle);
        QWidget::setTabOrder(cbxMeterStyle, cbxLanguage);
        QWidget::setTabOrder(cbxLanguage, spnMixerRows);
        QWidget::setTabOrder(spnMixerRows, chbAudioAlerts);
        QWidget::setTabOrder(chbAudioAlerts, cbxSoundcard);
        QWidget::setTabOrder(cbxSoundcard, butDriverSetup);
        QWidget::setTabOrder(butDriverSetup, cbxLInChan);
        QWidget::setTabOrder(cbxLInChan, cbxRInChan);
        QWidget::setTabOrder(cbxRInChan, cbxLOutChan);
        QWidget::setTabOrder(cbxLOutChan, cbxROutChan);
        QWidget::setTabOrder(cbxROutChan, cbxAudioChannels);
        QWidget::setTabOrder(cbxAudioChannels, cbxAudioQuality);
        QWidget::setTabOrder(cbxAudioQuality, rbtBufferDelayPreferred);
        QWidget::setTabOrder(rbtBufferDelayPreferred, rbtBufferDelayDefault);
        QWidget::setTabOrder(rbtBufferDelayDefault, rbtBufferDelaySafe);
        QWidget::setTabOrder(rbtBufferDelaySafe, chbAutoJitBuf);
        QWidget::setTabOrder(chbAutoJitBuf, sldNetBuf);
        QWidget::setTabOrder(sldNetBuf, sldNetBufServer);
        QWidget::setTabOrder(sldNetBufServer, chbSmallNetworkBuffers);
        QWidget::setTabOrder(chbSmallNetworkBuffers, cbxCustomDirectories);
        QWidget::setTabOrder(cbxCustomDirectories, tbtDeleteCustomDirectory);
        QWidget::setTabOrder(tbtDeleteCustomDirectory, edtNewClientLevel);
        QWidget::setTabOrder(edtNewClientLevel, cbxInputBoost);
        QWidget::setTabOrder(cbxInputBoost, chbDetectFeedback);
        QWidget::setTabOrder(chbDetectFeedback, sldAudioPan);
        QWidget::setTabOrder(sldAudioPan, grbMidiControls);
        QWidget::setTabOrder(grbMidiControls, cbxMidiDevice);
        QWidget::setTabOrder(cbxMidiDevice, cbxChannel);
        QWidget::setTabOrder(cbxChannel, chbMIDIPickupMode);
        QWidget::setTabOrder(chbMIDIPickupMode, grbMidiMuteMyself);
        QWidget::setTabOrder(grbMidiMuteMyself, spnMuteMyself);
        QWidget::setTabOrder(spnMuteMyself, butLearnMuteMyself);
        QWidget::setTabOrder(butLearnMuteMyself, grbMidiFader);
        QWidget::setTabOrder(grbMidiFader, spnFaderOffset);
        QWidget::setTabOrder(spnFaderOffset, butLearnFaderOffset);
        QWidget::setTabOrder(butLearnFaderOffset, spnFaderCount);
        QWidget::setTabOrder(spnFaderCount, grbMidiPan);
        QWidget::setTabOrder(grbMidiPan, spnPanOffset);
        QWidget::setTabOrder(spnPanOffset, butLearnPanOffset);
        QWidget::setTabOrder(butLearnPanOffset, spnPanCount);
        QWidget::setTabOrder(spnPanCount, grbMidiSolo);
        QWidget::setTabOrder(grbMidiSolo, spnSoloOffset);
        QWidget::setTabOrder(spnSoloOffset, butLearnSoloOffset);
        QWidget::setTabOrder(butLearnSoloOffset, spnSoloCount);
        QWidget::setTabOrder(spnSoloCount, grbMidiMute);
        QWidget::setTabOrder(grbMidiMute, spnMuteOffset);
        QWidget::setTabOrder(spnMuteOffset, butLearnMuteOffset);
        QWidget::setTabOrder(butLearnMuteOffset, spnMuteCount);

        retranslateUi(CClientSettingsDlgBase);

        tabSettings->setCurrentIndex(3);


        QMetaObject::connectSlotsByName(CClientSettingsDlgBase);
    } // setupUi

    void retranslateUi(QDialog *CClientSettingsDlgBase)
    {
        CClientSettingsDlgBase->setWindowTitle(QCoreApplication::translate("CClientSettingsDlgBase", "Settings", nullptr));
        grbMusicien->setTitle(QCoreApplication::translate("CClientSettingsDlgBase", "Musician's Profile", nullptr));
        plblAlias->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Alias/Name", nullptr));
        plblInstrument->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Instrument", nullptr));
        plblCountry->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Country/Region", nullptr));
        plblCity->setText(QCoreApplication::translate("CClientSettingsDlgBase", "City", nullptr));
        plblSkill->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Skill", nullptr));
        gbxUserInterface->setTitle(QCoreApplication::translate("CClientSettingsDlgBase", "User Interface", nullptr));
        lblSkin->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Skin", nullptr));
        lblMeterStyle->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Meter Style", nullptr));
        lblLanguage->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Language", nullptr));
        lblMixerRows->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Mixer Rows", nullptr));
        lblAudioAlerts->setText(QString());
        chbAudioAlerts->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Audio Alerts", nullptr));
        tabSettings->setTabText(tabSettings->indexOf(tab_0_User), QCoreApplication::translate("CClientSettingsDlgBase", "My Profile", nullptr));
        lblSoundcardDevice->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Audio Device", nullptr));
        butDriverSetup->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Driver Setup", nullptr));
        lblInChannelMapping->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Input Channel Mapping", nullptr));
        lblLInChan->setText(QCoreApplication::translate("CClientSettingsDlgBase", "L", nullptr));
        lblRInChan->setText(QCoreApplication::translate("CClientSettingsDlgBase", "R", nullptr));
        lblOutChannelMapping->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Output Channel Mapping", nullptr));
        lblLOutChan->setText(QCoreApplication::translate("CClientSettingsDlgBase", "L", nullptr));
        lblROutChan->setText(QCoreApplication::translate("CClientSettingsDlgBase", "R", nullptr));
        lblAudioChannels->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Audio Channels", nullptr));
        lblAudioQuality->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Audio Quality", nullptr));
        grbSoundCrdBufDelay->setTitle(QCoreApplication::translate("CClientSettingsDlgBase", "Buffer Delay", nullptr));
        rbtBufferDelayPreferred->setText(QCoreApplication::translate("CClientSettingsDlgBase", "(preferred)", nullptr));
        rbtBufferDelayDefault->setText(QCoreApplication::translate("CClientSettingsDlgBase", "(default)", nullptr));
        rbtBufferDelaySafe->setText(QCoreApplication::translate("CClientSettingsDlgBase", "(safe)", nullptr));
        grbJitterBuffer->setTitle(QCoreApplication::translate("CClientSettingsDlgBase", "Jitter Buffer", nullptr));
        chbAutoJitBuf->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Auto", nullptr));
        lblNetBufLabel->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Local", nullptr));
        lblNetBufServerLabel->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Server", nullptr));
        lblNetBuf->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Size", nullptr));
        lblNetBufServer->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Size", nullptr));
        chbSmallNetworkBuffers->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Small Network Buffers", nullptr));
        grbUpstreamValue->setTitle(QCoreApplication::translate("CClientSettingsDlgBase", "Audio Stream Rate", nullptr));
        lblUpstreamUnit->setText(QCoreApplication::translate("CClientSettingsDlgBase", "kbps", nullptr));
        lblUpstreamValue->setText(QCoreApplication::translate("CClientSettingsDlgBase", "val", nullptr));
        tabSettings->setTabText(tabSettings->indexOf(tab_1_AudioNet), QCoreApplication::translate("CClientSettingsDlgBase", "Audio/Network Setup", nullptr));
        lblCustomDirectories->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Custom Directories:", nullptr));
        lblNewClientLevel->setText(QCoreApplication::translate("CClientSettingsDlgBase", "New Client Level", nullptr));
        lblPercentUnit->setText(QCoreApplication::translate("CClientSettingsDlgBase", "%", nullptr));
        lblInputBoost->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Input Boost", nullptr));
        chbDetectFeedback->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Feedback Protection", nullptr));
        groupBox->setTitle(QCoreApplication::translate("CClientSettingsDlgBase", "Input Balance", nullptr));
        lblAudioPan->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Pan", nullptr));
        sldAudioPan->setStyleSheet(QString());
        lblAudioPanValue->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Center", nullptr));
        tabSettings->setTabText(tabSettings->indexOf(tab_2_advanced), QCoreApplication::translate("CClientSettingsDlgBase", "Advanced Setup", nullptr));
        grbMidiControls->setTitle(QCoreApplication::translate("CClientSettingsDlgBase", "MIDI-in", nullptr));
        lblChannel->setText(QCoreApplication::translate("CClientSettingsDlgBase", "MIDI Channel", nullptr));
        cbxChannel->setItemText(0, QCoreApplication::translate("CClientSettingsDlgBase", "0 (all)", nullptr));
        cbxChannel->setItemText(1, QCoreApplication::translate("CClientSettingsDlgBase", "1", nullptr));
        cbxChannel->setItemText(2, QCoreApplication::translate("CClientSettingsDlgBase", "2", nullptr));
        cbxChannel->setItemText(3, QCoreApplication::translate("CClientSettingsDlgBase", "3", nullptr));
        cbxChannel->setItemText(4, QCoreApplication::translate("CClientSettingsDlgBase", "4", nullptr));
        cbxChannel->setItemText(5, QCoreApplication::translate("CClientSettingsDlgBase", "5", nullptr));
        cbxChannel->setItemText(6, QCoreApplication::translate("CClientSettingsDlgBase", "6", nullptr));
        cbxChannel->setItemText(7, QCoreApplication::translate("CClientSettingsDlgBase", "7", nullptr));
        cbxChannel->setItemText(8, QCoreApplication::translate("CClientSettingsDlgBase", "8", nullptr));
        cbxChannel->setItemText(9, QCoreApplication::translate("CClientSettingsDlgBase", "9", nullptr));
        cbxChannel->setItemText(10, QCoreApplication::translate("CClientSettingsDlgBase", "10", nullptr));
        cbxChannel->setItemText(11, QCoreApplication::translate("CClientSettingsDlgBase", "11", nullptr));
        cbxChannel->setItemText(12, QCoreApplication::translate("CClientSettingsDlgBase", "12", nullptr));
        cbxChannel->setItemText(13, QCoreApplication::translate("CClientSettingsDlgBase", "13", nullptr));
        cbxChannel->setItemText(14, QCoreApplication::translate("CClientSettingsDlgBase", "14", nullptr));
        cbxChannel->setItemText(15, QCoreApplication::translate("CClientSettingsDlgBase", "15", nullptr));
        cbxChannel->setItemText(16, QCoreApplication::translate("CClientSettingsDlgBase", "16", nullptr));

        chbMIDIPickupMode->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Pick-up Mode", nullptr));
        grbMidiMuteMyself->setTitle(QCoreApplication::translate("CClientSettingsDlgBase", "Mute Myself", nullptr));
        lblMuteMyselfCC->setText(QCoreApplication::translate("CClientSettingsDlgBase", "MIDI CC", nullptr));
        butLearnMuteMyself->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Learn", nullptr));
        grbMidiFader->setTitle(QCoreApplication::translate("CClientSettingsDlgBase", "Fader", nullptr));
        lblFaderOffset->setText(QCoreApplication::translate("CClientSettingsDlgBase", "First MIDI CC", nullptr));
        butLearnFaderOffset->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Learn", nullptr));
        lblFaderCount->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Count", nullptr));
        grbMidiMute->setTitle(QCoreApplication::translate("CClientSettingsDlgBase", "Mute", nullptr));
        lblMuteOffset->setText(QCoreApplication::translate("CClientSettingsDlgBase", "First MIDI CC", nullptr));
        butLearnMuteOffset->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Learn", nullptr));
        lblMuteCount->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Count", nullptr));
        grbMidiSolo->setTitle(QCoreApplication::translate("CClientSettingsDlgBase", "Solo", nullptr));
        lblSoloOffset->setText(QCoreApplication::translate("CClientSettingsDlgBase", "First MIDI CC", nullptr));
        butLearnSoloOffset->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Learn", nullptr));
        lblSoloCount->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Count", nullptr));
        grbMidiPan->setTitle(QCoreApplication::translate("CClientSettingsDlgBase", "Pan", nullptr));
        lblPanOffset->setText(QCoreApplication::translate("CClientSettingsDlgBase", "First MIDI CC", nullptr));
        butLearnPanOffset->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Learn", nullptr));
        lblPanCount->setText(QCoreApplication::translate("CClientSettingsDlgBase", "Count", nullptr));
        lblMidiDevice->setText(QCoreApplication::translate("CClientSettingsDlgBase", "MIDI Device", nullptr));
        tabSettings->setTabText(tabSettings->indexOf(tab_3_midi), QCoreApplication::translate("CClientSettingsDlgBase", "MIDI Control", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CClientSettingsDlgBase: public Ui_CClientSettingsDlgBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CLIENTSETTINGSDLGBASE_H
