/****************************************************************************
** Meta object code from reading C++ file 'client.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/client.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'client.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7CClientE_t {};
} // unnamed namespace

template <> constexpr inline auto CClient::qt_create_metaobjectdata<qt_meta_tag_ZN7CClientE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CClient",
        "ConClientListMesReceived",
        "",
        "CVector<CChannelInfo>",
        "vecChanInfo",
        "ChatTextReceived",
        "strChatText",
        "ClientIDReceived",
        "iChanID",
        "MuteStateHasChangedReceived",
        "bIsMuted",
        "LicenceRequired",
        "ELicenceType",
        "eLicenceType",
        "VersionAndOSReceived",
        "COSUtil::EOpSystemType",
        "eOSType",
        "strVersion",
        "PingTimeReceived",
        "iPingTime",
        "RecorderStateReceived",
        "ERecorderState",
        "eRecorderState",
        "CLServerListReceived",
        "CHostAddress",
        "InetAddr",
        "CVector<CServerInfo>",
        "vecServerInfo",
        "CLRedServerListReceived",
        "CLConnClientsListMesReceived",
        "CLPingTimeWithNumClientsReceived",
        "iNumClients",
        "CLVersionAndOSReceived",
        "CLChannelLevelListReceived",
        "CVector<uint16_t>",
        "vecLevelList",
        "Disconnected",
        "SoundDeviceChanged",
        "strError",
        "ControllerInFaderLevel",
        "iChannelIdx",
        "iValue",
        "ControllerInPanValue",
        "ControllerInFaderIsSolo",
        "bIsSolo",
        "ControllerInFaderIsMute",
        "bIsMute",
        "ControllerInMuteMyself",
        "bMute",
        "MidiCCReceived",
        "ccNumber",
        "OnHandledSignal",
        "sigNum",
        "OnSendProtMessage",
        "CVector<uint8_t>",
        "vecMessage",
        "OnInvalidPacketReceived",
        "RecHostAddr",
        "OnDetectedCLMessage",
        "vecbyMesBodyData",
        "iRecID",
        "OnReqJittBufSize",
        "OnJittBufSizeChanged",
        "iNewJitBufSize",
        "OnReqChanInfo",
        "OnNewConnection",
        "OnCLDisconnection",
        "OnCLPingReceived",
        "iMs",
        "OnSendCLProtMessage",
        "OnCLPingWithNumClientsReceived",
        "OnSndCrdReinitRequest",
        "iSndCrdResetType",
        "OnControllerInFaderLevel",
        "OnControllerInPanValue",
        "OnControllerInFaderIsSolo",
        "OnControllerInFaderIsMute",
        "OnControllerInMuteMyself",
        "OnClientIDReceived",
        "iServerChanID",
        "OnMuteStateHasChangedReceived",
        "OnCLChannelLevelListReceived",
        "OnConClientListMesReceived",
        "OnMidiCCReceived"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ConClientListMesReceived'
        QtMocHelpers::SignalData<void(CVector<CChannelInfo>)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'ChatTextReceived'
        QtMocHelpers::SignalData<void(QString)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'ClientIDReceived'
        QtMocHelpers::SignalData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Signal 'MuteStateHasChangedReceived'
        QtMocHelpers::SignalData<void(int, bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { QMetaType::Bool, 10 },
        }}),
        // Signal 'LicenceRequired'
        QtMocHelpers::SignalData<void(ELicenceType)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'VersionAndOSReceived'
        QtMocHelpers::SignalData<void(COSUtil::EOpSystemType, QString)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { QMetaType::QString, 17 },
        }}),
        // Signal 'PingTimeReceived'
        QtMocHelpers::SignalData<void(int)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 19 },
        }}),
        // Signal 'RecorderStateReceived'
        QtMocHelpers::SignalData<void(ERecorderState)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Signal 'CLServerListReceived'
        QtMocHelpers::SignalData<void(CHostAddress, CVector<CServerInfo>)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 }, { 0x80000000 | 26, 27 },
        }}),
        // Signal 'CLRedServerListReceived'
        QtMocHelpers::SignalData<void(CHostAddress, CVector<CServerInfo>)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 }, { 0x80000000 | 26, 27 },
        }}),
        // Signal 'CLConnClientsListMesReceived'
        QtMocHelpers::SignalData<void(CHostAddress, CVector<CChannelInfo>)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 }, { 0x80000000 | 3, 4 },
        }}),
        // Signal 'CLPingTimeWithNumClientsReceived'
        QtMocHelpers::SignalData<void(CHostAddress, int, int)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 }, { QMetaType::Int, 19 }, { QMetaType::Int, 31 },
        }}),
        // Signal 'CLVersionAndOSReceived'
        QtMocHelpers::SignalData<void(CHostAddress, COSUtil::EOpSystemType, QString)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 }, { 0x80000000 | 15, 16 }, { QMetaType::QString, 17 },
        }}),
        // Signal 'CLChannelLevelListReceived'
        QtMocHelpers::SignalData<void(CHostAddress, CVector<uint16_t>)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 }, { 0x80000000 | 34, 35 },
        }}),
        // Signal 'Disconnected'
        QtMocHelpers::SignalData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'SoundDeviceChanged'
        QtMocHelpers::SignalData<void(QString)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 38 },
        }}),
        // Signal 'ControllerInFaderLevel'
        QtMocHelpers::SignalData<void(int, int)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 40 }, { QMetaType::Int, 41 },
        }}),
        // Signal 'ControllerInPanValue'
        QtMocHelpers::SignalData<void(int, int)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 40 }, { QMetaType::Int, 41 },
        }}),
        // Signal 'ControllerInFaderIsSolo'
        QtMocHelpers::SignalData<void(int, bool)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 40 }, { QMetaType::Bool, 44 },
        }}),
        // Signal 'ControllerInFaderIsMute'
        QtMocHelpers::SignalData<void(int, bool)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 40 }, { QMetaType::Bool, 46 },
        }}),
        // Signal 'ControllerInMuteMyself'
        QtMocHelpers::SignalData<void(bool)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 48 },
        }}),
        // Signal 'MidiCCReceived'
        QtMocHelpers::SignalData<void(int)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 50 },
        }}),
        // Slot 'OnHandledSignal'
        QtMocHelpers::SlotData<void(int)>(51, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 52 },
        }}),
        // Slot 'OnSendProtMessage'
        QtMocHelpers::SlotData<void(CVector<uint8_t>)>(53, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 54, 55 },
        }}),
        // Slot 'OnInvalidPacketReceived'
        QtMocHelpers::SlotData<void(CHostAddress)>(56, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 24, 57 },
        }}),
        // Slot 'OnDetectedCLMessage'
        QtMocHelpers::SlotData<void(CVector<uint8_t>, int, CHostAddress)>(58, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 54, 59 }, { QMetaType::Int, 60 }, { 0x80000000 | 24, 57 },
        }}),
        // Slot 'OnReqJittBufSize'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'OnJittBufSizeChanged'
        QtMocHelpers::SlotData<void(int)>(62, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 63 },
        }}),
        // Slot 'OnReqChanInfo'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'OnNewConnection'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'OnCLDisconnection'
        QtMocHelpers::SlotData<void(CHostAddress)>(66, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'OnCLPingReceived'
        QtMocHelpers::SlotData<void(CHostAddress, int)>(67, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 24, 25 }, { QMetaType::Int, 68 },
        }}),
        // Slot 'OnSendCLProtMessage'
        QtMocHelpers::SlotData<void(CHostAddress, CVector<uint8_t>)>(69, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 24, 25 }, { 0x80000000 | 54, 55 },
        }}),
        // Slot 'OnCLPingWithNumClientsReceived'
        QtMocHelpers::SlotData<void(CHostAddress, int, int)>(70, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 24, 25 }, { QMetaType::Int, 68 }, { QMetaType::Int, 31 },
        }}),
        // Slot 'OnSndCrdReinitRequest'
        QtMocHelpers::SlotData<void(int)>(71, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 72 },
        }}),
        // Slot 'OnControllerInFaderLevel'
        QtMocHelpers::SlotData<void(int, int)>(73, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 40 }, { QMetaType::Int, 41 },
        }}),
        // Slot 'OnControllerInPanValue'
        QtMocHelpers::SlotData<void(int, int)>(74, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 40 }, { QMetaType::Int, 41 },
        }}),
        // Slot 'OnControllerInFaderIsSolo'
        QtMocHelpers::SlotData<void(int, bool)>(75, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 40 }, { QMetaType::Bool, 44 },
        }}),
        // Slot 'OnControllerInFaderIsMute'
        QtMocHelpers::SlotData<void(int, bool)>(76, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 40 }, { QMetaType::Bool, 46 },
        }}),
        // Slot 'OnControllerInMuteMyself'
        QtMocHelpers::SlotData<void(bool)>(77, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 48 },
        }}),
        // Slot 'OnClientIDReceived'
        QtMocHelpers::SlotData<void(int)>(78, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 79 },
        }}),
        // Slot 'OnMuteStateHasChangedReceived'
        QtMocHelpers::SlotData<void(int, bool)>(80, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 79 }, { QMetaType::Bool, 10 },
        }}),
        // Slot 'OnCLChannelLevelListReceived'
        QtMocHelpers::SlotData<void(CHostAddress, CVector<uint16_t>)>(81, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 24, 25 }, { 0x80000000 | 34, 35 },
        }}),
        // Slot 'OnConClientListMesReceived'
        QtMocHelpers::SlotData<void(CVector<CChannelInfo>)>(82, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'OnMidiCCReceived'
        QtMocHelpers::SlotData<void(int)>(83, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 50 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CClient, qt_meta_tag_ZN7CClientE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CClient::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7CClientE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7CClientE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7CClientE_t>.metaTypes,
    nullptr
} };

void CClient::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CClient *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ConClientListMesReceived((*reinterpret_cast<std::add_pointer_t<CVector<CChannelInfo>>>(_a[1]))); break;
        case 1: _t->ChatTextReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->ClientIDReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->MuteStateHasChangedReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 4: _t->LicenceRequired((*reinterpret_cast<std::add_pointer_t<ELicenceType>>(_a[1]))); break;
        case 5: _t->VersionAndOSReceived((*reinterpret_cast<std::add_pointer_t<COSUtil::EOpSystemType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->PingTimeReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->RecorderStateReceived((*reinterpret_cast<std::add_pointer_t<ERecorderState>>(_a[1]))); break;
        case 8: _t->CLServerListReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CVector<CServerInfo>>>(_a[2]))); break;
        case 9: _t->CLRedServerListReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CVector<CServerInfo>>>(_a[2]))); break;
        case 10: _t->CLConnClientsListMesReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CVector<CChannelInfo>>>(_a[2]))); break;
        case 11: _t->CLPingTimeWithNumClientsReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 12: _t->CLVersionAndOSReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<COSUtil::EOpSystemType>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 13: _t->CLChannelLevelListReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CVector<uint16_t>>>(_a[2]))); break;
        case 14: _t->Disconnected(); break;
        case 15: _t->SoundDeviceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->ControllerInFaderLevel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 17: _t->ControllerInPanValue((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 18: _t->ControllerInFaderIsSolo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 19: _t->ControllerInFaderIsMute((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 20: _t->ControllerInMuteMyself((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->MidiCCReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 22: _t->OnHandledSignal((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 23: _t->OnSendProtMessage((*reinterpret_cast<std::add_pointer_t<CVector<uint8_t>>>(_a[1]))); break;
        case 24: _t->OnInvalidPacketReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 25: _t->OnDetectedCLMessage((*reinterpret_cast<std::add_pointer_t<CVector<uint8_t>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[3]))); break;
        case 26: _t->OnReqJittBufSize(); break;
        case 27: _t->OnJittBufSizeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 28: _t->OnReqChanInfo(); break;
        case 29: _t->OnNewConnection(); break;
        case 30: _t->OnCLDisconnection((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 31: _t->OnCLPingReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 32: _t->OnSendCLProtMessage((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CVector<uint8_t>>>(_a[2]))); break;
        case 33: _t->OnCLPingWithNumClientsReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 34: _t->OnSndCrdReinitRequest((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 35: _t->OnControllerInFaderLevel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 36: _t->OnControllerInPanValue((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 37: _t->OnControllerInFaderIsSolo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 38: _t->OnControllerInFaderIsMute((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 39: _t->OnControllerInMuteMyself((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 40: _t->OnClientIDReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 41: _t->OnMuteStateHasChangedReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 42: _t->OnCLChannelLevelListReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CVector<uint16_t>>>(_a[2]))); break;
        case 43: _t->OnConClientListMesReceived((*reinterpret_cast<std::add_pointer_t<CVector<CChannelInfo>>>(_a[1]))); break;
        case 44: _t->OnMidiCCReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(CVector<CChannelInfo> )>(_a, &CClient::ConClientListMesReceived, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(QString )>(_a, &CClient::ChatTextReceived, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(int )>(_a, &CClient::ClientIDReceived, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(int , bool )>(_a, &CClient::MuteStateHasChangedReceived, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(ELicenceType )>(_a, &CClient::LicenceRequired, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(COSUtil::EOpSystemType , QString )>(_a, &CClient::VersionAndOSReceived, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(int )>(_a, &CClient::PingTimeReceived, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(ERecorderState )>(_a, &CClient::RecorderStateReceived, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(CHostAddress , CVector<CServerInfo> )>(_a, &CClient::CLServerListReceived, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(CHostAddress , CVector<CServerInfo> )>(_a, &CClient::CLRedServerListReceived, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(CHostAddress , CVector<CChannelInfo> )>(_a, &CClient::CLConnClientsListMesReceived, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(CHostAddress , int , int )>(_a, &CClient::CLPingTimeWithNumClientsReceived, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(CHostAddress , COSUtil::EOpSystemType , QString )>(_a, &CClient::CLVersionAndOSReceived, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(CHostAddress , CVector<uint16_t> )>(_a, &CClient::CLChannelLevelListReceived, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)()>(_a, &CClient::Disconnected, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(QString )>(_a, &CClient::SoundDeviceChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(int , int )>(_a, &CClient::ControllerInFaderLevel, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(int , int )>(_a, &CClient::ControllerInPanValue, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(int , bool )>(_a, &CClient::ControllerInFaderIsSolo, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(int , bool )>(_a, &CClient::ControllerInFaderIsMute, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(bool )>(_a, &CClient::ControllerInMuteMyself, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClient::*)(int )>(_a, &CClient::MidiCCReceived, 21))
            return;
    }
}

const QMetaObject *CClient::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CClient::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7CClientE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CClient::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 45)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 45;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 45)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 45;
    }
    return _id;
}

// SIGNAL 0
void CClient::ConClientListMesReceived(CVector<CChannelInfo> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void CClient::ChatTextReceived(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void CClient::ClientIDReceived(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void CClient::MuteStateHasChangedReceived(int _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void CClient::LicenceRequired(ELicenceType _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void CClient::VersionAndOSReceived(COSUtil::EOpSystemType _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void CClient::PingTimeReceived(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void CClient::RecorderStateReceived(ERecorderState _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void CClient::CLServerListReceived(CHostAddress _t1, CVector<CServerInfo> _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void CClient::CLRedServerListReceived(CHostAddress _t1, CVector<CServerInfo> _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void CClient::CLConnClientsListMesReceived(CHostAddress _t1, CVector<CChannelInfo> _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2);
}

// SIGNAL 11
void CClient::CLPingTimeWithNumClientsReceived(CHostAddress _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2, _t3);
}

// SIGNAL 12
void CClient::CLVersionAndOSReceived(CHostAddress _t1, COSUtil::EOpSystemType _t2, QString _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2, _t3);
}

// SIGNAL 13
void CClient::CLChannelLevelListReceived(CHostAddress _t1, CVector<uint16_t> _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1, _t2);
}

// SIGNAL 14
void CClient::Disconnected()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void CClient::SoundDeviceChanged(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void CClient::ControllerInFaderLevel(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1, _t2);
}

// SIGNAL 17
void CClient::ControllerInPanValue(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1, _t2);
}

// SIGNAL 18
void CClient::ControllerInFaderIsSolo(int _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1, _t2);
}

// SIGNAL 19
void CClient::ControllerInFaderIsMute(int _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1, _t2);
}

// SIGNAL 20
void CClient::ControllerInMuteMyself(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void CClient::MidiCCReceived(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}
QT_WARNING_POP
