/******************************************************************************\
 * Copyright (c) 2004-2026
 *
 * Author(s):
 *  Volker Fischer
 *
 * As of Jamulus 3.12.1dev (commit eb172d47): All new source code contributions must be licensed
 * under AGPL 3.0 or any later version.
 *
 * Existing code: Code contributed before 3.12.1dev (commit eb172d47) was licensed under GPL 2.0+.
 * This code will be licensed under GPL 3.0 (or any later version) from
 * 3.12.1dev (commit eb172d47).  When distributed as part of Jamulus, the AGPL 3.0 terms govern
 * the combined work, including network use provisions.
 *
 ******************************************************************************
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * ---------------------------------------------------------------------------
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
\******************************************************************************/

#pragma once

#include <QCloseEvent>
#include <QLabel>
#include <QListView>
#include <QTimer>
#include <QPixmap>
#include <QThread>
#include <QSlider>
#include <QMenuBar>
#include <QLayout>
#include <QSystemTrayIcon>
#include <QSettings>
#include <QFileDialog>
#if QT_VERSION >= QT_VERSION_CHECK( 5, 6, 0 )
#    include <QVersionNumber>
#endif
#include "global.h"
#include "util.h"
#include "server.h"
#include "settings.h"
#include "ui_serverdlgbase.h"

/* Definitions ****************************************************************/
// update time for GUI controls
#define GUI_CONTRL_UPDATE_TIME 1000 // ms

// Strings used in multiple places
#define SREC_NOT_INITIALISED CServerDlg::tr ( "Not initialised" )
#define SREC_NOT_ENABLED     CServerDlg::tr ( "Not enabled" )
#define SREC_NOT_RECORDING   CServerDlg::tr ( "Not recording" )
#define SREC_RECORDING       CServerDlg::tr ( "Recording" )

/* Classes ********************************************************************/
class CServerDlg : public CBaseDlg, private Ui_CServerDlgBase
{
    Q_OBJECT

public:
    CServerDlg ( CServer* pNServP, CServerSettings* pNSetP, const bool bStartMinimized, QWidget* parent = nullptr );

protected:
    virtual void changeEvent ( QEvent* pEvent );
    virtual void closeEvent ( QCloseEvent* Event );

    void UpdateGUIDependencies();
    void UpdateSystemTrayIcon ( const bool bIsActive );
    void ShowWindowInForeground()
    {
        showNormal();
        raise();
        activateWindow();
    }
    void ModifyAutoStartEntry ( const bool bDoAutoStart );
    void UpdateRecorderStatus ( QString sessionDir );

    QTimer           Timer;
    CServer*         pServer;
    CServerSettings* pSettings;

    CVector<QTreeWidgetItem*> vecpListViewItems;
    QMutex                    ListViewMutex;

    QMenuBar* pMenu;

    bool            bSystemTrayIconAvailable;
    QSystemTrayIcon SystemTrayIcon;
    QPixmap         BitmapSystemTrayInactive;
    QPixmap         BitmapSystemTrayActive;
    QMenu*          pSystemTrayIconMenu;

public slots:
    // From the GUI
    void OnDirectoryTypeCurrentIndexChanged ( int iTypeIdx );
    void OnServerNameEditingFinished();
    void OnLocationCityEditingFinished();
    void OnLocationCountryCurrentIndexChanged ( int iCntryListItem );
    void OnEnableRecorderStateChanged ( int value ) { pServer->SetEnableRecording ( Qt::CheckState::Checked == value ); }
    void OnNewRecordingClicked() { pServer->RequestNewRecording(); }
    void OnWelcomeMessageChanged() { pServer->SetWelcomeMessage ( tedWelcomeMessage->toPlainText() ); }

    void OnLanguageChanged ( QString strLanguage ) { pSettings->strLanguage = strLanguage; }
    void OnCustomDirectoryEditingFinished();
    void OnRecordingDirClicked();
    void OnClearRecordingDirClicked();
    void OnServerListPersistenceClicked();
    void OnClearServerListPersistenceClicked();
    void OnStartOnOSStartStateChanged ( int value );
    void OnEnableDelayPanningStateChanged ( int value ) { pServer->SetEnableDelayPanning ( Qt::CheckState::Checked == value ); }

    void OnSysTrayMenuOpen() { ShowWindowInForeground(); }
    void OnSysTrayMenuHide() { hide(); }
    void OnSysTrayMenuExit() { close(); }
    void OnSysTrayActivated ( QSystemTrayIcon::ActivationReason ActReason );

    // From the Server
    void OnServerStarted();
    void OnServerStopped();
    void OnSvrRegStatusChanged() { UpdateGUIDependencies(); }
    void OnRecordingSessionStarted ( QString sessionDir ) { UpdateRecorderStatus ( sessionDir ); }
    void OnStopRecorder();
    void OnCLVersionAndOSReceived ( CHostAddress, COSUtil::EOpSystemType, QString strVersion );

    // Our timer
    void OnTimer();
};
