/********************************************************************************
** Form generated from reading UI file 'chatdlgbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CHATDLGBASE_H
#define UI_CHATDLGBASE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_CChatDlgBase
{
public:
    QVBoxLayout *vboxLayout;
    QTextBrowser *txvChatWindow;
    QHBoxLayout *horizontalLayout;
    QLineEdit *edtLocalInputText;
    QPushButton *butSend;

    void setupUi(QDialog *CChatDlgBase)
    {
        if (CChatDlgBase->objectName().isEmpty())
            CChatDlgBase->setObjectName("CChatDlgBase");
        CChatDlgBase->resize(435, 405);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(CChatDlgBase->sizePolicy().hasHeightForWidth());
        CChatDlgBase->setSizePolicy(sizePolicy);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/png/main/res/fronticon.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        CChatDlgBase->setWindowIcon(icon);
        CChatDlgBase->setSizeGripEnabled(true);
        vboxLayout = new QVBoxLayout(CChatDlgBase);
        vboxLayout->setObjectName("vboxLayout");
        txvChatWindow = new QTextBrowser(CChatDlgBase);
        txvChatWindow->setObjectName("txvChatWindow");
        txvChatWindow->setAcceptDrops(false);
        txvChatWindow->setTextInteractionFlags(Qt::LinksAccessibleByKeyboard|Qt::LinksAccessibleByMouse|Qt::TextBrowserInteraction|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);
        txvChatWindow->setOpenExternalLinks(false);
        txvChatWindow->setOpenLinks(false);

        vboxLayout->addWidget(txvChatWindow);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        edtLocalInputText = new QLineEdit(CChatDlgBase);
        edtLocalInputText->setObjectName("edtLocalInputText");

        horizontalLayout->addWidget(edtLocalInputText);

        butSend = new QPushButton(CChatDlgBase);
        butSend->setObjectName("butSend");
        butSend->setAutoDefault(true);

        horizontalLayout->addWidget(butSend);


        vboxLayout->addLayout(horizontalLayout);


        retranslateUi(CChatDlgBase);

        butSend->setDefault(true);


        QMetaObject::connectSlotsByName(CChatDlgBase);
    } // setupUi

    void retranslateUi(QDialog *CChatDlgBase)
    {
        CChatDlgBase->setWindowTitle(QCoreApplication::translate("CChatDlgBase", "Chat", nullptr));
        butSend->setText(QCoreApplication::translate("CChatDlgBase", "&Send", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CChatDlgBase: public Ui_CChatDlgBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CHATDLGBASE_H
