/****************************************************************************
** Meta object code from reading C++ file 'channel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/channel.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'channel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8CChannelE_t {};
} // unnamed namespace

template <> constexpr inline auto CChannel::qt_create_metaobjectdata<qt_meta_tag_ZN8CChannelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CChannel",
        "MessReadyForSending",
        "",
        "CVector<uint8_t>",
        "vecMessage",
        "NewConnection",
        "ReqJittBufSize",
        "JittBufSizeChanged",
        "iNewJitBufSize",
        "ServerAutoSockBufSizeChange",
        "iNNumFra",
        "ReqConnClientsList",
        "ConClientListMesReceived",
        "CVector<CChannelInfo>",
        "vecChanInfo",
        "ChanInfoHasChanged",
        "ClientIDReceived",
        "iChanID",
        "MuteStateHasChanged",
        "bIsMuted",
        "MuteStateHasChangedReceived",
        "ReqChanInfo",
        "ChatTextReceived",
        "strChatText",
        "ReqNetTranspProps",
        "LicenceRequired",
        "ELicenceType",
        "eLicenceType",
        "VersionAndOSReceived",
        "COSUtil::EOpSystemType",
        "eOSType",
        "strVersion",
        "RecorderStateReceived",
        "ERecorderState",
        "eRecorderState",
        "Disconnected",
        "DetectedCLMessage",
        "vecbyMesBodyData",
        "iRecID",
        "CHostAddress",
        "RecHostAddr",
        "ParseMessageBody",
        "iRecCounter",
        "OnSendProtMessage",
        "OnJittBufSizeChange",
        "OnChangeChanGain",
        "fNewGain",
        "OnChangeChanPan",
        "fNewPan",
        "OnChangeChanInfo",
        "CChannelCoreInfo",
        "ChanInfo",
        "OnNetTranspPropsReceived",
        "CNetworkTransportProps",
        "NetworkTransportProps",
        "OnReqNetTranspProps",
        "OnReqSplitMessSupport",
        "OnSplitMessSupported",
        "OnVersionAndOSReceived",
        "OnParseMessageBody",
        "OnProtocolMessageReceived",
        "OnProtocolCLMessageReceived",
        "OnNewConnection"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'MessReadyForSending'
        QtMocHelpers::SignalData<void(CVector<uint8_t>)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'NewConnection'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ReqJittBufSize'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'JittBufSizeChanged'
        QtMocHelpers::SignalData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Signal 'ServerAutoSockBufSizeChange'
        QtMocHelpers::SignalData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Signal 'ReqConnClientsList'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ConClientListMesReceived'
        QtMocHelpers::SignalData<void(CVector<CChannelInfo>)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'ChanInfoHasChanged'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ClientIDReceived'
        QtMocHelpers::SignalData<void(int)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 17 },
        }}),
        // Signal 'MuteStateHasChanged'
        QtMocHelpers::SignalData<void(int, bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 17 }, { QMetaType::Bool, 19 },
        }}),
        // Signal 'MuteStateHasChangedReceived'
        QtMocHelpers::SignalData<void(int, bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 17 }, { QMetaType::Bool, 19 },
        }}),
        // Signal 'ReqChanInfo'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ChatTextReceived'
        QtMocHelpers::SignalData<void(QString)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Signal 'ReqNetTranspProps'
        QtMocHelpers::SignalData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'LicenceRequired'
        QtMocHelpers::SignalData<void(ELicenceType)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Signal 'VersionAndOSReceived'
        QtMocHelpers::SignalData<void(COSUtil::EOpSystemType, QString)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 }, { QMetaType::QString, 31 },
        }}),
        // Signal 'RecorderStateReceived'
        QtMocHelpers::SignalData<void(ERecorderState)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Signal 'Disconnected'
        QtMocHelpers::SignalData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'DetectedCLMessage'
        QtMocHelpers::SignalData<void(CVector<uint8_t>, int, CHostAddress)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 37 }, { QMetaType::Int, 38 }, { 0x80000000 | 39, 40 },
        }}),
        // Signal 'ParseMessageBody'
        QtMocHelpers::SignalData<void(CVector<uint8_t>, int, int)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 37 }, { QMetaType::Int, 42 }, { QMetaType::Int, 38 },
        }}),
        // Slot 'OnSendProtMessage'
        QtMocHelpers::SlotData<void(CVector<uint8_t>)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'OnJittBufSizeChange'
        QtMocHelpers::SlotData<void(int)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'OnChangeChanGain'
        QtMocHelpers::SlotData<void(int, float)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 17 }, { QMetaType::Float, 46 },
        }}),
        // Slot 'OnChangeChanPan'
        QtMocHelpers::SlotData<void(int, float)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 17 }, { QMetaType::Float, 48 },
        }}),
        // Slot 'OnChangeChanInfo'
        QtMocHelpers::SlotData<void(CChannelCoreInfo)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 50, 51 },
        }}),
        // Slot 'OnNetTranspPropsReceived'
        QtMocHelpers::SlotData<void(CNetworkTransportProps)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 53, 54 },
        }}),
        // Slot 'OnReqNetTranspProps'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnReqSplitMessSupport'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnSplitMessSupported'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnVersionAndOSReceived'
        QtMocHelpers::SlotData<void(COSUtil::EOpSystemType, QString)>(58, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 }, { QMetaType::QString, 31 },
        }}),
        // Slot 'OnParseMessageBody'
        QtMocHelpers::SlotData<void(CVector<uint8_t>, int, int)>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 37 }, { QMetaType::Int, 42 }, { QMetaType::Int, 38 },
        }}),
        // Slot 'OnProtocolMessageReceived'
        QtMocHelpers::SlotData<void(int, int, CVector<uint8_t>, CHostAddress)>(60, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 42 }, { QMetaType::Int, 38 }, { 0x80000000 | 3, 37 }, { 0x80000000 | 39, 40 },
        }}),
        // Slot 'OnProtocolCLMessageReceived'
        QtMocHelpers::SlotData<void(int, CVector<uint8_t>, CHostAddress)>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 38 }, { 0x80000000 | 3, 37 }, { 0x80000000 | 39, 40 },
        }}),
        // Slot 'OnNewConnection'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CChannel, qt_meta_tag_ZN8CChannelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CChannel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8CChannelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8CChannelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8CChannelE_t>.metaTypes,
    nullptr
} };

void CChannel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CChannel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->MessReadyForSending((*reinterpret_cast<std::add_pointer_t<CVector<uint8_t>>>(_a[1]))); break;
        case 1: _t->NewConnection(); break;
        case 2: _t->ReqJittBufSize(); break;
        case 3: _t->JittBufSizeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->ServerAutoSockBufSizeChange((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->ReqConnClientsList(); break;
        case 6: _t->ConClientListMesReceived((*reinterpret_cast<std::add_pointer_t<CVector<CChannelInfo>>>(_a[1]))); break;
        case 7: _t->ChanInfoHasChanged(); break;
        case 8: _t->ClientIDReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->MuteStateHasChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 10: _t->MuteStateHasChangedReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 11: _t->ReqChanInfo(); break;
        case 12: _t->ChatTextReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->ReqNetTranspProps(); break;
        case 14: _t->LicenceRequired((*reinterpret_cast<std::add_pointer_t<ELicenceType>>(_a[1]))); break;
        case 15: _t->VersionAndOSReceived((*reinterpret_cast<std::add_pointer_t<COSUtil::EOpSystemType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 16: _t->RecorderStateReceived((*reinterpret_cast<std::add_pointer_t<ERecorderState>>(_a[1]))); break;
        case 17: _t->Disconnected(); break;
        case 18: _t->DetectedCLMessage((*reinterpret_cast<std::add_pointer_t<CVector<uint8_t>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[3]))); break;
        case 19: _t->ParseMessageBody((*reinterpret_cast<std::add_pointer_t<CVector<uint8_t>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 20: _t->OnSendProtMessage((*reinterpret_cast<std::add_pointer_t<CVector<uint8_t>>>(_a[1]))); break;
        case 21: _t->OnJittBufSizeChange((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 22: _t->OnChangeChanGain((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 23: _t->OnChangeChanPan((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 24: _t->OnChangeChanInfo((*reinterpret_cast<std::add_pointer_t<CChannelCoreInfo>>(_a[1]))); break;
        case 25: _t->OnNetTranspPropsReceived((*reinterpret_cast<std::add_pointer_t<CNetworkTransportProps>>(_a[1]))); break;
        case 26: _t->OnReqNetTranspProps(); break;
        case 27: _t->OnReqSplitMessSupport(); break;
        case 28: _t->OnSplitMessSupported(); break;
        case 29: _t->OnVersionAndOSReceived((*reinterpret_cast<std::add_pointer_t<COSUtil::EOpSystemType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 30: _t->OnParseMessageBody((*reinterpret_cast<std::add_pointer_t<CVector<uint8_t>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 31: _t->OnProtocolMessageReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<CVector<uint8_t>>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[4]))); break;
        case 32: _t->OnProtocolCLMessageReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CVector<uint8_t>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[3]))); break;
        case 33: _t->OnNewConnection(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CChannel::*)(CVector<uint8_t> )>(_a, &CChannel::MessReadyForSending, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CChannel::*)()>(_a, &CChannel::NewConnection, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CChannel::*)()>(_a, &CChannel::ReqJittBufSize, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CChannel::*)(int )>(_a, &CChannel::JittBufSizeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (CChannel::*)(int )>(_a, &CChannel::ServerAutoSockBufSizeChange, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (CChannel::*)()>(_a, &CChannel::ReqConnClientsList, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (CChannel::*)(CVector<CChannelInfo> )>(_a, &CChannel::ConClientListMesReceived, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (CChannel::*)()>(_a, &CChannel::ChanInfoHasChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (CChannel::*)(int )>(_a, &CChannel::ClientIDReceived, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (CChannel::*)(int , bool )>(_a, &CChannel::MuteStateHasChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (CChannel::*)(int , bool )>(_a, &CChannel::MuteStateHasChangedReceived, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (CChannel::*)()>(_a, &CChannel::ReqChanInfo, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (CChannel::*)(QString )>(_a, &CChannel::ChatTextReceived, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (CChannel::*)()>(_a, &CChannel::ReqNetTranspProps, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (CChannel::*)(ELicenceType )>(_a, &CChannel::LicenceRequired, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (CChannel::*)(COSUtil::EOpSystemType , QString )>(_a, &CChannel::VersionAndOSReceived, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (CChannel::*)(ERecorderState )>(_a, &CChannel::RecorderStateReceived, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (CChannel::*)()>(_a, &CChannel::Disconnected, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (CChannel::*)(CVector<uint8_t> , int , CHostAddress )>(_a, &CChannel::DetectedCLMessage, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (CChannel::*)(CVector<uint8_t> , int , int )>(_a, &CChannel::ParseMessageBody, 19))
            return;
    }
}

const QMetaObject *CChannel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CChannel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8CChannelE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CChannel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 34)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 34;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 34)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 34;
    }
    return _id;
}

// SIGNAL 0
void CChannel::MessReadyForSending(CVector<uint8_t> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void CChannel::NewConnection()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void CChannel::ReqJittBufSize()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void CChannel::JittBufSizeChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void CChannel::ServerAutoSockBufSizeChange(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void CChannel::ReqConnClientsList()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void CChannel::ConClientListMesReceived(CVector<CChannelInfo> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void CChannel::ChanInfoHasChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void CChannel::ClientIDReceived(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void CChannel::MuteStateHasChanged(int _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void CChannel::MuteStateHasChangedReceived(int _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2);
}

// SIGNAL 11
void CChannel::ReqChanInfo()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void CChannel::ChatTextReceived(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void CChannel::ReqNetTranspProps()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void CChannel::LicenceRequired(ELicenceType _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void CChannel::VersionAndOSReceived(COSUtil::EOpSystemType _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1, _t2);
}

// SIGNAL 16
void CChannel::RecorderStateReceived(ERecorderState _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void CChannel::Disconnected()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void CChannel::DetectedCLMessage(CVector<uint8_t> _t1, int _t2, CHostAddress _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1, _t2, _t3);
}

// SIGNAL 19
void CChannel::ParseMessageBody(CVector<uint8_t> _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
