/********************************************************************************
** Form generated from reading UI file 'aboutdlgbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ABOUTDLGBASE_H
#define UI_ABOUTDLGBASE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_CAboutDlgBase
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *pxlLogo;
    QVBoxLayout *vboxLayout1;
    QLabel *lblVersion;
    QLabel *lblCopyright;
    QSpacerItem *horizontalSpacer;
    QTabWidget *tabWidget;
    QWidget *tab;
    QHBoxLayout *horizontalLayout;
    QTextBrowser *txvAbout;
    QWidget *tab_2;
    QHBoxLayout *horizontalLayout_2;
    QTextBrowser *txvLibraries;
    QWidget *tab_3;
    QHBoxLayout *horizontalLayout_3;
    QTextBrowser *txvContributors;
    QWidget *tab_4;
    QHBoxLayout *horizontalLayout_4;
    QTextBrowser *txvTranslation;
    QHBoxLayout *hboxLayout1;
    QSpacerItem *spacerItem;
    QPushButton *buttonOk;

    void setupUi(QDialog *CAboutDlgBase)
    {
        if (CAboutDlgBase->objectName().isEmpty())
            CAboutDlgBase->setObjectName("CAboutDlgBase");
        CAboutDlgBase->resize(630, 465);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(CAboutDlgBase->sizePolicy().hasHeightForWidth());
        CAboutDlgBase->setSizePolicy(sizePolicy);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/png/main/res/fronticon.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        CAboutDlgBase->setWindowIcon(icon);
        CAboutDlgBase->setSizeGripEnabled(true);
        vboxLayout = new QVBoxLayout(CAboutDlgBase);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        pxlLogo = new QLabel(CAboutDlgBase);
        pxlLogo->setObjectName("pxlLogo");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(pxlLogo->sizePolicy().hasHeightForWidth());
        pxlLogo->setSizePolicy(sizePolicy1);
        pxlLogo->setPixmap(QPixmap(QString::fromUtf8(":/png/main/res/fronticon.png")));
        pxlLogo->setAlignment(Qt::AlignCenter);

        hboxLayout->addWidget(pxlLogo);

        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(0);
        vboxLayout1->setObjectName("vboxLayout1");
        lblVersion = new QLabel(CAboutDlgBase);
        lblVersion->setObjectName("lblVersion");
        lblVersion->setWordWrap(false);
        lblVersion->setMargin(2);

        vboxLayout1->addWidget(lblVersion);

        lblCopyright = new QLabel(CAboutDlgBase);
        lblCopyright->setObjectName("lblCopyright");
        lblCopyright->setWordWrap(false);
        lblCopyright->setMargin(2);

        vboxLayout1->addWidget(lblCopyright);


        hboxLayout->addLayout(vboxLayout1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(horizontalSpacer);


        vboxLayout->addLayout(hboxLayout);

        tabWidget = new QTabWidget(CAboutDlgBase);
        tabWidget->setObjectName("tabWidget");
        tab = new QWidget();
        tab->setObjectName("tab");
        horizontalLayout = new QHBoxLayout(tab);
        horizontalLayout->setObjectName("horizontalLayout");
        txvAbout = new QTextBrowser(tab);
        txvAbout->setObjectName("txvAbout");
        txvAbout->setOpenExternalLinks(true);

        horizontalLayout->addWidget(txvAbout);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        horizontalLayout_2 = new QHBoxLayout(tab_2);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        txvLibraries = new QTextBrowser(tab_2);
        txvLibraries->setObjectName("txvLibraries");
        txvLibraries->setOpenExternalLinks(true);

        horizontalLayout_2->addWidget(txvLibraries);

        tabWidget->addTab(tab_2, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName("tab_3");
        horizontalLayout_3 = new QHBoxLayout(tab_3);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        txvContributors = new QTextBrowser(tab_3);
        txvContributors->setObjectName("txvContributors");
        txvContributors->setOpenExternalLinks(true);

        horizontalLayout_3->addWidget(txvContributors);

        tabWidget->addTab(tab_3, QString());
        tab_4 = new QWidget();
        tab_4->setObjectName("tab_4");
        horizontalLayout_4 = new QHBoxLayout(tab_4);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        txvTranslation = new QTextBrowser(tab_4);
        txvTranslation->setObjectName("txvTranslation");
        txvTranslation->setOpenExternalLinks(true);

        horizontalLayout_4->addWidget(txvTranslation);

        tabWidget->addTab(tab_4, QString());

        vboxLayout->addWidget(tabWidget);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        spacerItem = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacerItem);

        buttonOk = new QPushButton(CAboutDlgBase);
        buttonOk->setObjectName("buttonOk");
        buttonOk->setAutoDefault(true);

        hboxLayout1->addWidget(buttonOk);


        vboxLayout->addLayout(hboxLayout1);


        retranslateUi(CAboutDlgBase);
        QObject::connect(buttonOk, &QPushButton::clicked, CAboutDlgBase, qOverload<>(&QDialog::accept));

        tabWidget->setCurrentIndex(0);
        buttonOk->setDefault(true);


        QMetaObject::connectSlotsByName(CAboutDlgBase);
    } // setupUi

    void retranslateUi(QDialog *CAboutDlgBase)
    {
        CAboutDlgBase->setWindowTitle(QCoreApplication::translate("CAboutDlgBase", "About", nullptr));
        pxlLogo->setText(QString());
        lblVersion->setText(QCoreApplication::translate("CAboutDlgBase", "TextLabelVersion", nullptr));
        lblCopyright->setText(QCoreApplication::translate("CAboutDlgBase", "Copyright \302\251 2005-2026 The Jamulus Development Team", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("CAboutDlgBase", "A&bout", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), QCoreApplication::translate("CAboutDlgBase", "&Libraries", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_3), QCoreApplication::translate("CAboutDlgBase", "&Contributors", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_4), QCoreApplication::translate("CAboutDlgBase", "&Translation", nullptr));
        buttonOk->setText(QCoreApplication::translate("CAboutDlgBase", "&OK", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CAboutDlgBase: public Ui_CAboutDlgBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ABOUTDLGBASE_H
