/********************************************************************************
** Form generated from reading UI file 'serverdlgbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SERVERDLGBASE_H
#define UI_SERVERDLGBASE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "util.h"

QT_BEGIN_NAMESPACE

class Ui_CServerDlgBase
{
public:
    QVBoxLayout *vboxLayout;
    QTreeWidget *lvwClients;
    QTabWidget *tabWidget;
    QWidget *tabServerSetup;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *hboxLayout;
    QLabel *lblDirectoryType;
    QComboBox *cbxDirectoryType;
    QLabel *lblRegSvrStatus;
    QSpacerItem *spacerItem;
    QGroupBox *grbServerInfo;
    QHBoxLayout *hboxLayout1;
    QVBoxLayout *vboxLayout1;
    QLabel *lblServerName;
    QLabel *lblLocationCity;
    QLabel *lblLocationCountry;
    QVBoxLayout *vboxLayout2;
    QLineEdit *edtServerName;
    QLineEdit *edtLocationCity;
    QComboBox *cbxLocationCountry;
    QHBoxLayout *hboxLayout2;
    QCheckBox *chbJamRecorder;
    QPushButton *pbtNewRecording;
    QLabel *lblRecorderStatus;
    QHBoxLayout *hboxLayout3;
    QLabel *lblCurrentSessionDir;
    QLineEdit *edtCurrentSessionDir;
    QLabel *label_2;
    QTextEdit *tedWelcomeMessage;
    QWidget *tabOptions;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *lblLanguage;
    CLanguageComboBox *cbxLanguage;
    QHBoxLayout *hboxLayout4;
    QPushButton *pbtRecordingDir;
    QLineEdit *edtRecordingDir;
    QToolButton *tbtClearRecordingDir;
    QHBoxLayout *hboxLayout5;
    QLabel *lblCustomDirectory;
    QLineEdit *edtCustomDirectory;
    QHBoxLayout *hboxLayout6;
    QPushButton *pbtServerListPersistence;
    QLineEdit *edtServerListPersistence;
    QToolButton *tbtClearServerListPersistence;
    QCheckBox *chbStartOnOSStart;
    QCheckBox *chbDelayPanning;
    QSpacerItem *verticalSpacer;
    QLabel *lblUpdateCheck;

    void setupUi(QDialog *CServerDlgBase)
    {
        if (CServerDlgBase->objectName().isEmpty())
            CServerDlgBase->setObjectName("CServerDlgBase");
        CServerDlgBase->resize(588, 560);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/png/main/res/fronticonserver.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        CServerDlgBase->setWindowIcon(icon);
        CServerDlgBase->setSizeGripEnabled(true);
        vboxLayout = new QVBoxLayout(CServerDlgBase);
        vboxLayout->setObjectName("vboxLayout");
        lvwClients = new QTreeWidget(CServerDlgBase);
        lvwClients->setObjectName("lvwClients");
        lvwClients->setRootIsDecorated(false);
        lvwClients->setColumnCount(4);

        vboxLayout->addWidget(lvwClients);

        tabWidget = new QTabWidget(CServerDlgBase);
        tabWidget->setObjectName("tabWidget");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(tabWidget->sizePolicy().hasHeightForWidth());
        tabWidget->setSizePolicy(sizePolicy);
        tabServerSetup = new QWidget();
        tabServerSetup->setObjectName("tabServerSetup");
        verticalLayout_2 = new QVBoxLayout(tabServerSetup);
        verticalLayout_2->setObjectName("verticalLayout_2");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        lblDirectoryType = new QLabel(tabServerSetup);
        lblDirectoryType->setObjectName("lblDirectoryType");

        hboxLayout->addWidget(lblDirectoryType);

        cbxDirectoryType = new QComboBox(tabServerSetup);
        cbxDirectoryType->setObjectName("cbxDirectoryType");

        hboxLayout->addWidget(cbxDirectoryType);

        lblRegSvrStatus = new QLabel(tabServerSetup);
        lblRegSvrStatus->setObjectName("lblRegSvrStatus");

        hboxLayout->addWidget(lblRegSvrStatus);

        spacerItem = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);


        verticalLayout_2->addLayout(hboxLayout);

        grbServerInfo = new QGroupBox(tabServerSetup);
        grbServerInfo->setObjectName("grbServerInfo");
        hboxLayout1 = new QHBoxLayout(grbServerInfo);
        hboxLayout1->setObjectName("hboxLayout1");
        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setObjectName("vboxLayout1");
        lblServerName = new QLabel(grbServerInfo);
        lblServerName->setObjectName("lblServerName");

        vboxLayout1->addWidget(lblServerName);

        lblLocationCity = new QLabel(grbServerInfo);
        lblLocationCity->setObjectName("lblLocationCity");

        vboxLayout1->addWidget(lblLocationCity);

        lblLocationCountry = new QLabel(grbServerInfo);
        lblLocationCountry->setObjectName("lblLocationCountry");

        vboxLayout1->addWidget(lblLocationCountry);


        hboxLayout1->addLayout(vboxLayout1);

        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setObjectName("vboxLayout2");
        edtServerName = new QLineEdit(grbServerInfo);
        edtServerName->setObjectName("edtServerName");

        vboxLayout2->addWidget(edtServerName);

        edtLocationCity = new QLineEdit(grbServerInfo);
        edtLocationCity->setObjectName("edtLocationCity");

        vboxLayout2->addWidget(edtLocationCity);

        cbxLocationCountry = new QComboBox(grbServerInfo);
        cbxLocationCountry->setObjectName("cbxLocationCountry");

        vboxLayout2->addWidget(cbxLocationCountry);


        hboxLayout1->addLayout(vboxLayout2);


        verticalLayout_2->addWidget(grbServerInfo);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setObjectName("hboxLayout2");
        chbJamRecorder = new QCheckBox(tabServerSetup);
        chbJamRecorder->setObjectName("chbJamRecorder");

        hboxLayout2->addWidget(chbJamRecorder);

        pbtNewRecording = new QPushButton(tabServerSetup);
        pbtNewRecording->setObjectName("pbtNewRecording");

        hboxLayout2->addWidget(pbtNewRecording);

        lblRecorderStatus = new QLabel(tabServerSetup);
        lblRecorderStatus->setObjectName("lblRecorderStatus");
        sizePolicy.setHeightForWidth(lblRecorderStatus->sizePolicy().hasHeightForWidth());
        lblRecorderStatus->setSizePolicy(sizePolicy);

        hboxLayout2->addWidget(lblRecorderStatus);


        verticalLayout_2->addLayout(hboxLayout2);

        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setObjectName("hboxLayout3");
        lblCurrentSessionDir = new QLabel(tabServerSetup);
        lblCurrentSessionDir->setObjectName("lblCurrentSessionDir");

        hboxLayout3->addWidget(lblCurrentSessionDir);

        edtCurrentSessionDir = new QLineEdit(tabServerSetup);
        edtCurrentSessionDir->setObjectName("edtCurrentSessionDir");
        edtCurrentSessionDir->setReadOnly(true);

        hboxLayout3->addWidget(edtCurrentSessionDir);


        verticalLayout_2->addLayout(hboxLayout3);

        label_2 = new QLabel(tabServerSetup);
        label_2->setObjectName("label_2");

        verticalLayout_2->addWidget(label_2);

        tedWelcomeMessage = new QTextEdit(tabServerSetup);
        tedWelcomeMessage->setObjectName("tedWelcomeMessage");
        sizePolicy.setHeightForWidth(tedWelcomeMessage->sizePolicy().hasHeightForWidth());
        tedWelcomeMessage->setSizePolicy(sizePolicy);

        verticalLayout_2->addWidget(tedWelcomeMessage);

        tabWidget->addTab(tabServerSetup, QString());
        tabOptions = new QWidget();
        tabOptions->setObjectName("tabOptions");
        verticalLayout = new QVBoxLayout(tabOptions);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        lblLanguage = new QLabel(tabOptions);
        lblLanguage->setObjectName("lblLanguage");

        horizontalLayout->addWidget(lblLanguage);

        cbxLanguage = new CLanguageComboBox(tabOptions);
        cbxLanguage->setObjectName("cbxLanguage");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(cbxLanguage->sizePolicy().hasHeightForWidth());
        cbxLanguage->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(cbxLanguage);


        verticalLayout->addLayout(horizontalLayout);

        hboxLayout4 = new QHBoxLayout();
        hboxLayout4->setObjectName("hboxLayout4");
        pbtRecordingDir = new QPushButton(tabOptions);
        pbtRecordingDir->setObjectName("pbtRecordingDir");

        hboxLayout4->addWidget(pbtRecordingDir);

        edtRecordingDir = new QLineEdit(tabOptions);
        edtRecordingDir->setObjectName("edtRecordingDir");
        edtRecordingDir->setReadOnly(true);

        hboxLayout4->addWidget(edtRecordingDir);

        tbtClearRecordingDir = new QToolButton(tabOptions);
        tbtClearRecordingDir->setObjectName("tbtClearRecordingDir");

        hboxLayout4->addWidget(tbtClearRecordingDir);


        verticalLayout->addLayout(hboxLayout4);

        hboxLayout5 = new QHBoxLayout();
        hboxLayout5->setObjectName("hboxLayout5");
        lblCustomDirectory = new QLabel(tabOptions);
        lblCustomDirectory->setObjectName("lblCustomDirectory");

        hboxLayout5->addWidget(lblCustomDirectory);

        edtCustomDirectory = new QLineEdit(tabOptions);
        edtCustomDirectory->setObjectName("edtCustomDirectory");

        hboxLayout5->addWidget(edtCustomDirectory);


        verticalLayout->addLayout(hboxLayout5);

        hboxLayout6 = new QHBoxLayout();
        hboxLayout6->setObjectName("hboxLayout6");
        pbtServerListPersistence = new QPushButton(tabOptions);
        pbtServerListPersistence->setObjectName("pbtServerListPersistence");

        hboxLayout6->addWidget(pbtServerListPersistence);

        edtServerListPersistence = new QLineEdit(tabOptions);
        edtServerListPersistence->setObjectName("edtServerListPersistence");
        edtServerListPersistence->setReadOnly(true);

        hboxLayout6->addWidget(edtServerListPersistence);

        tbtClearServerListPersistence = new QToolButton(tabOptions);
        tbtClearServerListPersistence->setObjectName("tbtClearServerListPersistence");

        hboxLayout6->addWidget(tbtClearServerListPersistence);


        verticalLayout->addLayout(hboxLayout6);

        chbStartOnOSStart = new QCheckBox(tabOptions);
        chbStartOnOSStart->setObjectName("chbStartOnOSStart");

        verticalLayout->addWidget(chbStartOnOSStart);

        chbDelayPanning = new QCheckBox(tabOptions);
        chbDelayPanning->setObjectName("chbDelayPanning");

        verticalLayout->addWidget(chbDelayPanning);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        tabWidget->addTab(tabOptions, QString());

        vboxLayout->addWidget(tabWidget);

        lblUpdateCheck = new QLabel(CServerDlgBase);
        lblUpdateCheck->setObjectName("lblUpdateCheck");

        vboxLayout->addWidget(lblUpdateCheck);

        QWidget::setTabOrder(lvwClients, tabWidget);
        QWidget::setTabOrder(tabWidget, cbxDirectoryType);
        QWidget::setTabOrder(cbxDirectoryType, edtServerName);
        QWidget::setTabOrder(edtServerName, edtLocationCity);
        QWidget::setTabOrder(edtLocationCity, cbxLocationCountry);
        QWidget::setTabOrder(cbxLocationCountry, chbJamRecorder);
        QWidget::setTabOrder(chbJamRecorder, edtCurrentSessionDir);
        QWidget::setTabOrder(edtCurrentSessionDir, pbtNewRecording);
        QWidget::setTabOrder(pbtNewRecording, tedWelcomeMessage);
        QWidget::setTabOrder(tedWelcomeMessage, cbxLanguage);
        QWidget::setTabOrder(cbxLanguage, pbtRecordingDir);
        QWidget::setTabOrder(pbtRecordingDir, edtRecordingDir);
        QWidget::setTabOrder(edtRecordingDir, tbtClearRecordingDir);
        QWidget::setTabOrder(tbtClearRecordingDir, edtCustomDirectory);
        QWidget::setTabOrder(edtCustomDirectory, pbtServerListPersistence);
        QWidget::setTabOrder(pbtServerListPersistence, edtServerListPersistence);
        QWidget::setTabOrder(edtServerListPersistence, tbtClearServerListPersistence);
        QWidget::setTabOrder(tbtClearServerListPersistence, chbStartOnOSStart);
        QWidget::setTabOrder(chbStartOnOSStart, chbDelayPanning);

        retranslateUi(CServerDlgBase);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(CServerDlgBase);
    } // setupUi

    void retranslateUi(QDialog *CServerDlgBase)
    {
        CServerDlgBase->setWindowTitle(QString());
        QTreeWidgetItem *___qtreewidgetitem = lvwClients->headerItem();
        ___qtreewidgetitem->setText(3, QCoreApplication::translate("CServerDlgBase", "Channels", nullptr));
        ___qtreewidgetitem->setText(2, QCoreApplication::translate("CServerDlgBase", "Jitter Buffer Size", nullptr));
        ___qtreewidgetitem->setText(1, QCoreApplication::translate("CServerDlgBase", "Name", nullptr));
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("CServerDlgBase", "Client IP:Port", nullptr));
        lblDirectoryType->setText(QCoreApplication::translate("CServerDlgBase", "Directory", nullptr));
        lblRegSvrStatus->setText(QCoreApplication::translate("CServerDlgBase", "STATUS", nullptr));
        grbServerInfo->setTitle(QCoreApplication::translate("CServerDlgBase", "My Server Info", nullptr));
        lblServerName->setText(QCoreApplication::translate("CServerDlgBase", "Name", nullptr));
        lblLocationCity->setText(QCoreApplication::translate("CServerDlgBase", "Location: City", nullptr));
        lblLocationCountry->setText(QCoreApplication::translate("CServerDlgBase", "Location: Region", nullptr));
        chbJamRecorder->setText(QCoreApplication::translate("CServerDlgBase", "Jam Recorder", nullptr));
        pbtNewRecording->setText(QCoreApplication::translate("CServerDlgBase", "New Recording", nullptr));
        lblRecorderStatus->setText(QCoreApplication::translate("CServerDlgBase", "STATUS", nullptr));
        lblCurrentSessionDir->setText(QCoreApplication::translate("CServerDlgBase", "Session", nullptr));
        label_2->setText(QCoreApplication::translate("CServerDlgBase", "Chat Window Welcome (HTML/CSS Supported)", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabServerSetup), QCoreApplication::translate("CServerDlgBase", "Server Setup", nullptr));
        lblLanguage->setText(QCoreApplication::translate("CServerDlgBase", "Language", nullptr));
        pbtRecordingDir->setText(QCoreApplication::translate("CServerDlgBase", "Recording Directory", nullptr));
        lblCustomDirectory->setText(QCoreApplication::translate("CServerDlgBase", "Custom Directory address", nullptr));
        pbtServerListPersistence->setText(QCoreApplication::translate("CServerDlgBase", "Server List Filename", nullptr));
        chbStartOnOSStart->setText(QCoreApplication::translate("CServerDlgBase", "Start Minimized on Windows Start", nullptr));
        chbDelayPanning->setText(QCoreApplication::translate("CServerDlgBase", "Delay panning", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabOptions), QCoreApplication::translate("CServerDlgBase", "Options", nullptr));
        lblUpdateCheck->setText(QCoreApplication::translate("CServerDlgBase", "Update check", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CServerDlgBase: public Ui_CServerDlgBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SERVERDLGBASE_H
