/****************************************************************************
** Meta object code from reading C++ file 'socket.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/socket.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'socket.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7CSocketE_t {};
} // unnamed namespace

template <> constexpr inline auto CSocket::qt_create_metaobjectdata<qt_meta_tag_ZN7CSocketE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CSocket",
        "NewConnection",
        "",
        "iChID",
        "iTotChans",
        "CHostAddress",
        "RecHostAddr",
        "ServerFull",
        "InvalidPacketReceived",
        "ProtocolMessageReceived",
        "iRecCounter",
        "iRecID",
        "CVector<uint8_t>",
        "vecbyMesBodyData",
        "HostAdr",
        "ProtocolCLMessageReceived"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'NewConnection'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'NewConnection'
        QtMocHelpers::SignalData<void(int, int, CHostAddress)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'ServerFull'
        QtMocHelpers::SignalData<void(CHostAddress)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'InvalidPacketReceived'
        QtMocHelpers::SignalData<void(CHostAddress)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'ProtocolMessageReceived'
        QtMocHelpers::SignalData<void(int, int, CVector<uint8_t>, CHostAddress)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { QMetaType::Int, 11 }, { 0x80000000 | 12, 13 }, { 0x80000000 | 5, 14 },
        }}),
        // Signal 'ProtocolCLMessageReceived'
        QtMocHelpers::SignalData<void(int, CVector<uint8_t>, CHostAddress)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { 0x80000000 | 12, 13 }, { 0x80000000 | 5, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CSocket, qt_meta_tag_ZN7CSocketE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CSocket::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7CSocketE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7CSocketE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7CSocketE_t>.metaTypes,
    nullptr
} };

void CSocket::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CSocket *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->NewConnection(); break;
        case 1: _t->NewConnection((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[3]))); break;
        case 2: _t->ServerFull((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 3: _t->InvalidPacketReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 4: _t->ProtocolMessageReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<CVector<uint8_t>>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[4]))); break;
        case 5: _t->ProtocolCLMessageReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CVector<uint8_t>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CSocket::*)()>(_a, &CSocket::NewConnection, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CSocket::*)(int , int , CHostAddress )>(_a, &CSocket::NewConnection, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CSocket::*)(CHostAddress )>(_a, &CSocket::ServerFull, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CSocket::*)(CHostAddress )>(_a, &CSocket::InvalidPacketReceived, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (CSocket::*)(int , int , CVector<uint8_t> , CHostAddress )>(_a, &CSocket::ProtocolMessageReceived, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (CSocket::*)(int , CVector<uint8_t> , CHostAddress )>(_a, &CSocket::ProtocolCLMessageReceived, 5))
            return;
    }
}

const QMetaObject *CSocket::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CSocket::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7CSocketE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CSocket::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void CSocket::NewConnection()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void CSocket::NewConnection(int _t1, int _t2, CHostAddress _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void CSocket::ServerFull(CHostAddress _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void CSocket::InvalidPacketReceived(CHostAddress _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void CSocket::ProtocolMessageReceived(int _t1, int _t2, CVector<uint8_t> _t3, CHostAddress _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 5
void CSocket::ProtocolCLMessageReceived(int _t1, CVector<uint8_t> _t2, CHostAddress _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3);
}
namespace {
struct qt_meta_tag_ZN15CHighPrioSocketE_t {};
} // unnamed namespace

template <> constexpr inline auto CHighPrioSocket::qt_create_metaobjectdata<qt_meta_tag_ZN15CHighPrioSocketE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CHighPrioSocket",
        "InvalidPacketReceived",
        "",
        "CHostAddress",
        "RecHostAddr"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'InvalidPacketReceived'
        QtMocHelpers::SignalData<void(CHostAddress)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CHighPrioSocket, qt_meta_tag_ZN15CHighPrioSocketE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CHighPrioSocket::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15CHighPrioSocketE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15CHighPrioSocketE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15CHighPrioSocketE_t>.metaTypes,
    nullptr
} };

void CHighPrioSocket::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CHighPrioSocket *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->InvalidPacketReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CHighPrioSocket::*)(CHostAddress )>(_a, &CHighPrioSocket::InvalidPacketReceived, 0))
            return;
    }
}

const QMetaObject *CHighPrioSocket::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CHighPrioSocket::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15CHighPrioSocketE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CHighPrioSocket::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void CHighPrioSocket::InvalidPacketReceived(CHostAddress _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
