/****************************************************************************
** Meta object code from reading C++ file 'connectdlg.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/connectdlg.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'connectdlg.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11CConnectDlgE_t {};
} // unnamed namespace

template <> constexpr inline auto CConnectDlg::qt_create_metaobjectdata<qt_meta_tag_ZN11CConnectDlgE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CConnectDlg",
        "ReqServerListQuery",
        "",
        "CHostAddress",
        "InetAddr",
        "CreateCLServerListPingMes",
        "CreateCLServerListReqVerAndOSMes",
        "CreateCLServerListReqConnClientsListMes",
        "OnServerListItemDoubleClicked",
        "QTreeWidgetItem*",
        "Item",
        "OnDirectoryChanged",
        "iTypeIdx",
        "OnFilterTextEdited",
        "OnExpandAllStateChanged",
        "value",
        "OnCustomDirectoriesChanged",
        "OnConnectClicked",
        "OnDeleteServerAddrClicked",
        "OnTimerPing",
        "OnTimerReRequestServList",
        "OnCurrentServerItemChanged",
        "current",
        "previous"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ReqServerListQuery'
        QtMocHelpers::SignalData<void(CHostAddress)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'CreateCLServerListPingMes'
        QtMocHelpers::SignalData<void(CHostAddress)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'CreateCLServerListReqVerAndOSMes'
        QtMocHelpers::SignalData<void(CHostAddress)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'CreateCLServerListReqConnClientsListMes'
        QtMocHelpers::SignalData<void(CHostAddress)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'OnServerListItemDoubleClicked'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *, int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Int, 2 },
        }}),
        // Slot 'OnDirectoryChanged'
        QtMocHelpers::SlotData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'OnFilterTextEdited'
        QtMocHelpers::SlotData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'OnExpandAllStateChanged'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'OnCustomDirectoriesChanged'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnConnectClicked'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnDeleteServerAddrClicked'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnTimerPing'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnTimerReRequestServList'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnCurrentServerItemChanged'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *, QTreeWidgetItem *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 22 }, { 0x80000000 | 9, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CConnectDlg, qt_meta_tag_ZN11CConnectDlgE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CConnectDlg::staticMetaObject = { {
    QMetaObject::SuperData::link<CBaseDlg::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CConnectDlgE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CConnectDlgE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11CConnectDlgE_t>.metaTypes,
    nullptr
} };

void CConnectDlg::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CConnectDlg *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ReqServerListQuery((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 1: _t->CreateCLServerListPingMes((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 2: _t->CreateCLServerListReqVerAndOSMes((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 3: _t->CreateCLServerListReqConnClientsListMes((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 4: _t->OnServerListItemDoubleClicked((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 5: _t->OnDirectoryChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->OnFilterTextEdited((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->OnExpandAllStateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->OnCustomDirectoriesChanged(); break;
        case 9: _t->OnConnectClicked(); break;
        case 10: _t->OnDeleteServerAddrClicked(); break;
        case 11: _t->OnTimerPing(); break;
        case 12: _t->OnTimerReRequestServList(); break;
        case 13: _t->OnCurrentServerItemChanged((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CConnectDlg::*)(CHostAddress )>(_a, &CConnectDlg::ReqServerListQuery, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CConnectDlg::*)(CHostAddress )>(_a, &CConnectDlg::CreateCLServerListPingMes, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CConnectDlg::*)(CHostAddress )>(_a, &CConnectDlg::CreateCLServerListReqVerAndOSMes, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CConnectDlg::*)(CHostAddress )>(_a, &CConnectDlg::CreateCLServerListReqConnClientsListMes, 3))
            return;
    }
}

const QMetaObject *CConnectDlg::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CConnectDlg::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CConnectDlgE_t>.strings))
        return static_cast<void*>(this);
    return CBaseDlg::qt_metacast(_clname);
}

int CConnectDlg::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CBaseDlg::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void CConnectDlg::ReqServerListQuery(CHostAddress _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void CConnectDlg::CreateCLServerListPingMes(CHostAddress _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void CConnectDlg::CreateCLServerListReqVerAndOSMes(CHostAddress _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void CConnectDlg::CreateCLServerListReqConnClientsListMes(CHostAddress _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
