/****************************************************************************
** Meta object code from reading C++ file 'serverdlg.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/serverdlg.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'serverdlg.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10CServerDlgE_t {};
} // unnamed namespace

template <> constexpr inline auto CServerDlg::qt_create_metaobjectdata<qt_meta_tag_ZN10CServerDlgE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CServerDlg",
        "OnDirectoryTypeCurrentIndexChanged",
        "",
        "iTypeIdx",
        "OnServerNameEditingFinished",
        "OnLocationCityEditingFinished",
        "OnLocationCountryCurrentIndexChanged",
        "iCntryListItem",
        "OnEnableRecorderStateChanged",
        "value",
        "OnNewRecordingClicked",
        "OnWelcomeMessageChanged",
        "OnLanguageChanged",
        "strLanguage",
        "OnCustomDirectoryEditingFinished",
        "OnRecordingDirClicked",
        "OnClearRecordingDirClicked",
        "OnServerListPersistenceClicked",
        "OnClearServerListPersistenceClicked",
        "OnStartOnOSStartStateChanged",
        "OnEnableDelayPanningStateChanged",
        "OnSysTrayMenuOpen",
        "OnSysTrayMenuHide",
        "OnSysTrayMenuExit",
        "OnSysTrayActivated",
        "QSystemTrayIcon::ActivationReason",
        "ActReason",
        "OnServerStarted",
        "OnServerStopped",
        "OnSvrRegStatusChanged",
        "OnRecordingSessionStarted",
        "sessionDir",
        "OnStopRecorder",
        "OnCLVersionAndOSReceived",
        "CHostAddress",
        "COSUtil::EOpSystemType",
        "strVersion",
        "OnTimer"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'OnDirectoryTypeCurrentIndexChanged'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'OnServerNameEditingFinished'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnLocationCityEditingFinished'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnLocationCountryCurrentIndexChanged'
        QtMocHelpers::SlotData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'OnEnableRecorderStateChanged'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'OnNewRecordingClicked'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnWelcomeMessageChanged'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnLanguageChanged'
        QtMocHelpers::SlotData<void(QString)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'OnCustomDirectoryEditingFinished'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnRecordingDirClicked'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnClearRecordingDirClicked'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnServerListPersistenceClicked'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnClearServerListPersistenceClicked'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnStartOnOSStartStateChanged'
        QtMocHelpers::SlotData<void(int)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'OnEnableDelayPanningStateChanged'
        QtMocHelpers::SlotData<void(int)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'OnSysTrayMenuOpen'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnSysTrayMenuHide'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnSysTrayMenuExit'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnSysTrayActivated'
        QtMocHelpers::SlotData<void(QSystemTrayIcon::ActivationReason)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Slot 'OnServerStarted'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnServerStopped'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnSvrRegStatusChanged'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnRecordingSessionStarted'
        QtMocHelpers::SlotData<void(QString)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 31 },
        }}),
        // Slot 'OnStopRecorder'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnCLVersionAndOSReceived'
        QtMocHelpers::SlotData<void(CHostAddress, COSUtil::EOpSystemType, QString)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 34, 2 }, { 0x80000000 | 35, 2 }, { QMetaType::QString, 36 },
        }}),
        // Slot 'OnTimer'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CServerDlg, qt_meta_tag_ZN10CServerDlgE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CServerDlg::staticMetaObject = { {
    QMetaObject::SuperData::link<CBaseDlg::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CServerDlgE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CServerDlgE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10CServerDlgE_t>.metaTypes,
    nullptr
} };

void CServerDlg::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CServerDlg *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->OnDirectoryTypeCurrentIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->OnServerNameEditingFinished(); break;
        case 2: _t->OnLocationCityEditingFinished(); break;
        case 3: _t->OnLocationCountryCurrentIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->OnEnableRecorderStateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->OnNewRecordingClicked(); break;
        case 6: _t->OnWelcomeMessageChanged(); break;
        case 7: _t->OnLanguageChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->OnCustomDirectoryEditingFinished(); break;
        case 9: _t->OnRecordingDirClicked(); break;
        case 10: _t->OnClearRecordingDirClicked(); break;
        case 11: _t->OnServerListPersistenceClicked(); break;
        case 12: _t->OnClearServerListPersistenceClicked(); break;
        case 13: _t->OnStartOnOSStartStateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 14: _t->OnEnableDelayPanningStateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->OnSysTrayMenuOpen(); break;
        case 16: _t->OnSysTrayMenuHide(); break;
        case 17: _t->OnSysTrayMenuExit(); break;
        case 18: _t->OnSysTrayActivated((*reinterpret_cast<std::add_pointer_t<QSystemTrayIcon::ActivationReason>>(_a[1]))); break;
        case 19: _t->OnServerStarted(); break;
        case 20: _t->OnServerStopped(); break;
        case 21: _t->OnSvrRegStatusChanged(); break;
        case 22: _t->OnRecordingSessionStarted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 23: _t->OnStopRecorder(); break;
        case 24: _t->OnCLVersionAndOSReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<COSUtil::EOpSystemType>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 25: _t->OnTimer(); break;
        default: ;
        }
    }
}

const QMetaObject *CServerDlg::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CServerDlg::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CServerDlgE_t>.strings))
        return static_cast<void*>(this);
    return CBaseDlg::qt_metacast(_clname);
}

int CServerDlg::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CBaseDlg::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 26;
    }
    return _id;
}
QT_WARNING_POP
