/****************************************************************************
** Meta object code from reading C++ file 'protocol.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/protocol.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'protocol.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9CProtocolE_t {};
} // unnamed namespace

template <> constexpr inline auto CProtocol::qt_create_metaobjectdata<qt_meta_tag_ZN9CProtocolE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CProtocol",
        "MessReadyForSending",
        "",
        "CVector<uint8_t>",
        "vecMessage",
        "CLMessReadyForSending",
        "CHostAddress",
        "InetAddr",
        "ChangeJittBufSize",
        "iNewJitBufSize",
        "ReqJittBufSize",
        "ChangeNetwBlSiFact",
        "iNewNetwBlSiFact",
        "ClientIDReceived",
        "iChanID",
        "ChangeChanGain",
        "fNewGain",
        "ChangeChanPan",
        "fNewPan",
        "MuteStateHasChangedReceived",
        "iCurID",
        "bIsMuted",
        "ConClientListMesReceived",
        "CVector<CChannelInfo>",
        "vecChanInfo",
        "ServerFullMesReceived",
        "ReqConnClientsList",
        "ChangeChanInfo",
        "CChannelCoreInfo",
        "ChanInfo",
        "ReqChanInfo",
        "ChatTextReceived",
        "strChatText",
        "NetTranspPropsReceived",
        "CNetworkTransportProps",
        "NetworkTransportProps",
        "ReqNetTranspProps",
        "ReqSplitMessSupport",
        "SplitMessSupported",
        "LicenceRequired",
        "ELicenceType",
        "eLicenceType",
        "VersionAndOSReceived",
        "COSUtil::EOpSystemType",
        "eOSType",
        "strVersion",
        "RecorderStateReceived",
        "ERecorderState",
        "eRecorderState",
        "CLPingReceived",
        "iMs",
        "CLPingWithNumClientsReceived",
        "iNumClients",
        "CLRegisterServerReceived",
        "LInetAddr",
        "CServerCoreInfo",
        "ServerInfo",
        "CLRegisterServerExReceived",
        "CLUnregisterServerReceived",
        "CLServerListReceived",
        "CVector<CServerInfo>",
        "vecServerInfo",
        "CLRedServerListReceived",
        "CLReqServerList",
        "CLSendEmptyMes",
        "TargetInetAddr",
        "CLDisconnection",
        "CLVersionAndOSReceived",
        "CLReqVersionAndOS",
        "CLConnClientsListMesReceived",
        "CLReqConnClientsList",
        "CLChannelLevelListReceived",
        "CVector<uint16_t>",
        "vecLevelList",
        "CLRegisterServerResp",
        "ESvrRegResult",
        "eStatus",
        "OnTimerSendMess"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'MessReadyForSending'
        QtMocHelpers::SignalData<void(CVector<uint8_t>)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'CLMessReadyForSending'
        QtMocHelpers::SignalData<void(CHostAddress, CVector<uint8_t>)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 3, 4 },
        }}),
        // Signal 'ChangeJittBufSize'
        QtMocHelpers::SignalData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Signal 'ReqJittBufSize'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ChangeNetwBlSiFact'
        QtMocHelpers::SignalData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Signal 'ClientIDReceived'
        QtMocHelpers::SignalData<void(int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Signal 'ChangeChanGain'
        QtMocHelpers::SignalData<void(int, float)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 }, { QMetaType::Float, 16 },
        }}),
        // Signal 'ChangeChanPan'
        QtMocHelpers::SignalData<void(int, float)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 }, { QMetaType::Float, 18 },
        }}),
        // Signal 'MuteStateHasChangedReceived'
        QtMocHelpers::SignalData<void(int, bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 20 }, { QMetaType::Bool, 21 },
        }}),
        // Signal 'ConClientListMesReceived'
        QtMocHelpers::SignalData<void(CVector<CChannelInfo>)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Signal 'ServerFullMesReceived'
        QtMocHelpers::SignalData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ReqConnClientsList'
        QtMocHelpers::SignalData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ChangeChanInfo'
        QtMocHelpers::SignalData<void(CChannelCoreInfo)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Signal 'ReqChanInfo'
        QtMocHelpers::SignalData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ChatTextReceived'
        QtMocHelpers::SignalData<void(QString)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 32 },
        }}),
        // Signal 'NetTranspPropsReceived'
        QtMocHelpers::SignalData<void(CNetworkTransportProps)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 34, 35 },
        }}),
        // Signal 'ReqNetTranspProps'
        QtMocHelpers::SignalData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ReqSplitMessSupport'
        QtMocHelpers::SignalData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'SplitMessSupported'
        QtMocHelpers::SignalData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'LicenceRequired'
        QtMocHelpers::SignalData<void(ELicenceType)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 40, 41 },
        }}),
        // Signal 'VersionAndOSReceived'
        QtMocHelpers::SignalData<void(COSUtil::EOpSystemType, QString)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 43, 44 }, { QMetaType::QString, 45 },
        }}),
        // Signal 'RecorderStateReceived'
        QtMocHelpers::SignalData<void(ERecorderState)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 47, 48 },
        }}),
        // Signal 'CLPingReceived'
        QtMocHelpers::SignalData<void(CHostAddress, int)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { QMetaType::Int, 50 },
        }}),
        // Signal 'CLPingWithNumClientsReceived'
        QtMocHelpers::SignalData<void(CHostAddress, int, int)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { QMetaType::Int, 50 }, { QMetaType::Int, 52 },
        }}),
        // Signal 'CLRegisterServerReceived'
        QtMocHelpers::SignalData<void(CHostAddress, CHostAddress, CServerCoreInfo)>(53, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 6, 54 }, { 0x80000000 | 55, 56 },
        }}),
        // Signal 'CLRegisterServerExReceived'
        QtMocHelpers::SignalData<void(CHostAddress, CHostAddress, CServerCoreInfo, COSUtil::EOpSystemType, QString)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 6, 54 }, { 0x80000000 | 55, 56 }, { 0x80000000 | 43, 44 },
            { QMetaType::QString, 45 },
        }}),
        // Signal 'CLUnregisterServerReceived'
        QtMocHelpers::SignalData<void(CHostAddress)>(58, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'CLServerListReceived'
        QtMocHelpers::SignalData<void(CHostAddress, CVector<CServerInfo>)>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 60, 61 },
        }}),
        // Signal 'CLRedServerListReceived'
        QtMocHelpers::SignalData<void(CHostAddress, CVector<CServerInfo>)>(62, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 60, 61 },
        }}),
        // Signal 'CLReqServerList'
        QtMocHelpers::SignalData<void(CHostAddress)>(63, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'CLSendEmptyMes'
        QtMocHelpers::SignalData<void(CHostAddress)>(64, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 65 },
        }}),
        // Signal 'CLDisconnection'
        QtMocHelpers::SignalData<void(CHostAddress)>(66, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'CLVersionAndOSReceived'
        QtMocHelpers::SignalData<void(CHostAddress, COSUtil::EOpSystemType, QString)>(67, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 43, 44 }, { QMetaType::QString, 45 },
        }}),
        // Signal 'CLReqVersionAndOS'
        QtMocHelpers::SignalData<void(CHostAddress)>(68, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'CLConnClientsListMesReceived'
        QtMocHelpers::SignalData<void(CHostAddress, CVector<CChannelInfo>)>(69, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 23, 24 },
        }}),
        // Signal 'CLReqConnClientsList'
        QtMocHelpers::SignalData<void(CHostAddress)>(70, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'CLChannelLevelListReceived'
        QtMocHelpers::SignalData<void(CHostAddress, CVector<uint16_t>)>(71, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 72, 73 },
        }}),
        // Signal 'CLRegisterServerResp'
        QtMocHelpers::SignalData<void(CHostAddress, ESvrRegResult)>(74, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 75, 76 },
        }}),
        // Slot 'OnTimerSendMess'
        QtMocHelpers::SlotData<void()>(77, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CProtocol, qt_meta_tag_ZN9CProtocolE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CProtocol::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9CProtocolE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9CProtocolE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9CProtocolE_t>.metaTypes,
    nullptr
} };

void CProtocol::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CProtocol *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->MessReadyForSending((*reinterpret_cast<std::add_pointer_t<CVector<uint8_t>>>(_a[1]))); break;
        case 1: _t->CLMessReadyForSending((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CVector<uint8_t>>>(_a[2]))); break;
        case 2: _t->ChangeJittBufSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->ReqJittBufSize(); break;
        case 4: _t->ChangeNetwBlSiFact((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->ClientIDReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->ChangeChanGain((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 7: _t->ChangeChanPan((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 8: _t->MuteStateHasChangedReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 9: _t->ConClientListMesReceived((*reinterpret_cast<std::add_pointer_t<CVector<CChannelInfo>>>(_a[1]))); break;
        case 10: _t->ServerFullMesReceived(); break;
        case 11: _t->ReqConnClientsList(); break;
        case 12: _t->ChangeChanInfo((*reinterpret_cast<std::add_pointer_t<CChannelCoreInfo>>(_a[1]))); break;
        case 13: _t->ReqChanInfo(); break;
        case 14: _t->ChatTextReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->NetTranspPropsReceived((*reinterpret_cast<std::add_pointer_t<CNetworkTransportProps>>(_a[1]))); break;
        case 16: _t->ReqNetTranspProps(); break;
        case 17: _t->ReqSplitMessSupport(); break;
        case 18: _t->SplitMessSupported(); break;
        case 19: _t->LicenceRequired((*reinterpret_cast<std::add_pointer_t<ELicenceType>>(_a[1]))); break;
        case 20: _t->VersionAndOSReceived((*reinterpret_cast<std::add_pointer_t<COSUtil::EOpSystemType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 21: _t->RecorderStateReceived((*reinterpret_cast<std::add_pointer_t<ERecorderState>>(_a[1]))); break;
        case 22: _t->CLPingReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 23: _t->CLPingWithNumClientsReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 24: _t->CLRegisterServerReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<CServerCoreInfo>>(_a[3]))); break;
        case 25: _t->CLRegisterServerExReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<CServerCoreInfo>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<COSUtil::EOpSystemType>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5]))); break;
        case 26: _t->CLUnregisterServerReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 27: _t->CLServerListReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CVector<CServerInfo>>>(_a[2]))); break;
        case 28: _t->CLRedServerListReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CVector<CServerInfo>>>(_a[2]))); break;
        case 29: _t->CLReqServerList((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 30: _t->CLSendEmptyMes((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 31: _t->CLDisconnection((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 32: _t->CLVersionAndOSReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<COSUtil::EOpSystemType>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 33: _t->CLReqVersionAndOS((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 34: _t->CLConnClientsListMesReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CVector<CChannelInfo>>>(_a[2]))); break;
        case 35: _t->CLReqConnClientsList((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 36: _t->CLChannelLevelListReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CVector<uint16_t>>>(_a[2]))); break;
        case 37: _t->CLRegisterServerResp((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ESvrRegResult>>(_a[2]))); break;
        case 38: _t->OnTimerSendMess(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CVector<uint8_t> )>(_a, &CProtocol::MessReadyForSending, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CHostAddress , CVector<uint8_t> )>(_a, &CProtocol::CLMessReadyForSending, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(int )>(_a, &CProtocol::ChangeJittBufSize, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)()>(_a, &CProtocol::ReqJittBufSize, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(int )>(_a, &CProtocol::ChangeNetwBlSiFact, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(int )>(_a, &CProtocol::ClientIDReceived, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(int , float )>(_a, &CProtocol::ChangeChanGain, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(int , float )>(_a, &CProtocol::ChangeChanPan, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(int , bool )>(_a, &CProtocol::MuteStateHasChangedReceived, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CVector<CChannelInfo> )>(_a, &CProtocol::ConClientListMesReceived, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)()>(_a, &CProtocol::ServerFullMesReceived, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)()>(_a, &CProtocol::ReqConnClientsList, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CChannelCoreInfo )>(_a, &CProtocol::ChangeChanInfo, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)()>(_a, &CProtocol::ReqChanInfo, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(QString )>(_a, &CProtocol::ChatTextReceived, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CNetworkTransportProps )>(_a, &CProtocol::NetTranspPropsReceived, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)()>(_a, &CProtocol::ReqNetTranspProps, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)()>(_a, &CProtocol::ReqSplitMessSupport, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)()>(_a, &CProtocol::SplitMessSupported, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(ELicenceType )>(_a, &CProtocol::LicenceRequired, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(COSUtil::EOpSystemType , QString )>(_a, &CProtocol::VersionAndOSReceived, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(ERecorderState )>(_a, &CProtocol::RecorderStateReceived, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CHostAddress , int )>(_a, &CProtocol::CLPingReceived, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CHostAddress , int , int )>(_a, &CProtocol::CLPingWithNumClientsReceived, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CHostAddress , CHostAddress , CServerCoreInfo )>(_a, &CProtocol::CLRegisterServerReceived, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CHostAddress , CHostAddress , CServerCoreInfo , COSUtil::EOpSystemType , QString )>(_a, &CProtocol::CLRegisterServerExReceived, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CHostAddress )>(_a, &CProtocol::CLUnregisterServerReceived, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CHostAddress , CVector<CServerInfo> )>(_a, &CProtocol::CLServerListReceived, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CHostAddress , CVector<CServerInfo> )>(_a, &CProtocol::CLRedServerListReceived, 28))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CHostAddress )>(_a, &CProtocol::CLReqServerList, 29))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CHostAddress )>(_a, &CProtocol::CLSendEmptyMes, 30))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CHostAddress )>(_a, &CProtocol::CLDisconnection, 31))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CHostAddress , COSUtil::EOpSystemType , QString )>(_a, &CProtocol::CLVersionAndOSReceived, 32))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CHostAddress )>(_a, &CProtocol::CLReqVersionAndOS, 33))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CHostAddress , CVector<CChannelInfo> )>(_a, &CProtocol::CLConnClientsListMesReceived, 34))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CHostAddress )>(_a, &CProtocol::CLReqConnClientsList, 35))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CHostAddress , CVector<uint16_t> )>(_a, &CProtocol::CLChannelLevelListReceived, 36))
            return;
        if (QtMocHelpers::indexOfMethod<void (CProtocol::*)(CHostAddress , ESvrRegResult )>(_a, &CProtocol::CLRegisterServerResp, 37))
            return;
    }
}

const QMetaObject *CProtocol::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CProtocol::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9CProtocolE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CProtocol::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 39)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 39;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 39)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 39;
    }
    return _id;
}

// SIGNAL 0
void CProtocol::MessReadyForSending(CVector<uint8_t> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void CProtocol::CLMessReadyForSending(CHostAddress _t1, CVector<uint8_t> _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void CProtocol::ChangeJittBufSize(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void CProtocol::ReqJittBufSize()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void CProtocol::ChangeNetwBlSiFact(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void CProtocol::ClientIDReceived(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void CProtocol::ChangeChanGain(int _t1, float _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void CProtocol::ChangeChanPan(int _t1, float _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void CProtocol::MuteStateHasChangedReceived(int _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void CProtocol::ConClientListMesReceived(CVector<CChannelInfo> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void CProtocol::ServerFullMesReceived()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void CProtocol::ReqConnClientsList()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void CProtocol::ChangeChanInfo(CChannelCoreInfo _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void CProtocol::ReqChanInfo()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void CProtocol::ChatTextReceived(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void CProtocol::NetTranspPropsReceived(CNetworkTransportProps _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void CProtocol::ReqNetTranspProps()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void CProtocol::ReqSplitMessSupport()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void CProtocol::SplitMessSupported()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void CProtocol::LicenceRequired(ELicenceType _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void CProtocol::VersionAndOSReceived(COSUtil::EOpSystemType _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1, _t2);
}

// SIGNAL 21
void CProtocol::RecorderStateReceived(ERecorderState _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}

// SIGNAL 22
void CProtocol::CLPingReceived(CHostAddress _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1, _t2);
}

// SIGNAL 23
void CProtocol::CLPingWithNumClientsReceived(CHostAddress _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1, _t2, _t3);
}

// SIGNAL 24
void CProtocol::CLRegisterServerReceived(CHostAddress _t1, CHostAddress _t2, CServerCoreInfo _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 24, nullptr, _t1, _t2, _t3);
}

// SIGNAL 25
void CProtocol::CLRegisterServerExReceived(CHostAddress _t1, CHostAddress _t2, CServerCoreInfo _t3, COSUtil::EOpSystemType _t4, QString _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 25, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 26
void CProtocol::CLUnregisterServerReceived(CHostAddress _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 26, nullptr, _t1);
}

// SIGNAL 27
void CProtocol::CLServerListReceived(CHostAddress _t1, CVector<CServerInfo> _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 27, nullptr, _t1, _t2);
}

// SIGNAL 28
void CProtocol::CLRedServerListReceived(CHostAddress _t1, CVector<CServerInfo> _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 28, nullptr, _t1, _t2);
}

// SIGNAL 29
void CProtocol::CLReqServerList(CHostAddress _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 29, nullptr, _t1);
}

// SIGNAL 30
void CProtocol::CLSendEmptyMes(CHostAddress _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 30, nullptr, _t1);
}

// SIGNAL 31
void CProtocol::CLDisconnection(CHostAddress _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 31, nullptr, _t1);
}

// SIGNAL 32
void CProtocol::CLVersionAndOSReceived(CHostAddress _t1, COSUtil::EOpSystemType _t2, QString _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 32, nullptr, _t1, _t2, _t3);
}

// SIGNAL 33
void CProtocol::CLReqVersionAndOS(CHostAddress _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 33, nullptr, _t1);
}

// SIGNAL 34
void CProtocol::CLConnClientsListMesReceived(CHostAddress _t1, CVector<CChannelInfo> _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 34, nullptr, _t1, _t2);
}

// SIGNAL 35
void CProtocol::CLReqConnClientsList(CHostAddress _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 35, nullptr, _t1);
}

// SIGNAL 36
void CProtocol::CLChannelLevelListReceived(CHostAddress _t1, CVector<uint16_t> _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 36, nullptr, _t1, _t2);
}

// SIGNAL 37
void CProtocol::CLRegisterServerResp(CHostAddress _t1, ESvrRegResult _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 37, nullptr, _t1, _t2);
}
QT_WARNING_POP
